/* $Workfile: T2p5x4Ss7.c $       $Revision: 11 $      $Modtime: 12/05/07 3:23p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc PublicDemo

  @module

    T2P5X4SS7.C

    A simple example console application demonstrating the use of OTX Driver.

    The applications works with the following board combinations:

      Thor-2(8)-PCI + Vidar-5x4-ASM,

    or OTX PCI Plus series boards,

      Gimle-16-PCI-Plus / Thor-2(4,8)-PCI-Plus + Vidar-5x4-ASM-CST,
      Gimle-16-PCI-Plus / Thor-2(4,8)-PCI-Plus + Vidar-5x4-ASM-PRO,
      Gimle-16-PCI-Plus / Thor-2(4,8)-PCI-Plus + Vidar-5x4-ASM-EX,
      Gimle-16-PCI-Plus / Thor-2(4,8)-PCI-Plus + Vidar-55x4-ASM,

    or OTX PCI Express series boards,

      Thor-2-PCI-Express

    or Thor-2-PCMCIA series boards,

      Thor-2-PCMCIA, 
      Thor-2-PCMCIA-Plus, 
      Thor-2-PCMCIA-Ex, 
      Thor-2-PCMCIA-Pro,
      Thor-2-PCMCIA-CST

    or Sleipnir-1-PCI.

    The sample application provides a simple console interface which utilizes
    simple keystrokes as commands.

    It configures all available E1T1 devices on the board. The E1T1 spans should be
    looped with an external loop cable (e.g. Li0<->Li1, Li2<->Li3, etc).
                                               
    Li0 sends MSUs to Li1. Li1 acknowledges the MSU.
    Li1 sends MSUs to Li0. Li0 acknowledges the MSU.

    SS7 FISU filtering is enabled.

    Press 'h' or '?' for help. 'q' to quit.

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#if !defined(_WIN32_WCE)
#include <assert.h>
#include <time.h>
#endif  // #if !defined(_WIN32_WCE)

#include "OsaStd.h"
#include "OsaCon.h"
#include "OsaThrd.h"
//#include "OtxGeneric.h"
#include "OtxErr.h"
#include "OtxBrd.h"
#include "OtxDef.h"
#include "OtxType.h"
#include "OtxDrv.h"
#include "OtxTone.h"
#include "OtxDsp.h"
#include "OtxHdlc.h"
#include "OtxT1E1.h"
#include "OtxData.h"

#define DEMO_TASK_REF 1L               // Task Reference for a non-blocking call

#define DEMO_DEFAULT_SS7_SIM_FRAMES 1000

#define DEMO_DEFAULT_LOG_FILE "T2p5x4Ss7.log"
#define DEMO_DEFAULT_BOARD_NO 0
#define DEMO_HDLC_RX_BUFFER_SIZE 2048

typedef struct DemoEnvT {
  char                  *pszInputStr;
  OTX_BOOL                bExitOnError;
  char                  *logFileName;
  OtxPhysicalDeviceTypeE nBoardType;
  char                   cwd[OTX_MAX_PATH];
  int                    coDsp;
  int                    coLi;
  int                    coHdlcPerLi;
  int                    nSimFrames;
  OtxT1E1LiModeE         eT1E1Mode;    // T1 or E1 mode
  OTX_BOOL               bSs7FisuFiltering;
  OTX_BOOL               bHdlcBitInversion;
  int                    boardNo;
  OTX_BOOL               bUseDspDebugger;
  OTX_BOOL               b56kbpsChannel;
  int                    nTimeSlots;
  OtxT1LineCodeE         eT1LineCode;
  OtxT1FrameFormatE      eT1FrameFormat;
  OtxE1LineCodeE         eE1LineCode;
  OtxE1FrameFormatE      eE1FrameFormat;
  OtxDspSpmE             eProgramModule;      // Standard Program Module to be loaded in the DSP (from the Lib)
  OTX_BOOL               bUseSenders;
  OTX_BOOL               bUseReceivers;
  OtxBrdClockSourceE  eClkSource;
} DemoEnvT;

void *DemoMsuTransmitFunction(
  void *lpParameter   // thread data
);

/*******************************************************************************
 Function Prototypes
*******************************************************************************/
static void DemoCreateEventQueue(void);
static void DemoOpenPhysicalDevices(void);
static void DemoConfigureE1T1Devices(void);
static void DemoEnablePhysicalDevices(void);
static void DemoCreateLogicalDevices(void);
static void DemoConnectDevices(void);
static void DemoInitializeLogicalDevices(void);
static void DemoEnableLogicalDevices(void);
static void DemoInitT1E1ConfigData(OUT OtxE1ConfigOptionsS *pE1CfgData, OUT OtxT1ConfigOptionsS *pT1CfgData);
static void DemoPrintEvent(IN OtxEventDataS *pEventData);

static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP);
static void DemoHelpPanel(void);
static void DemoLogf(char *formatStr, ...);
static void DemoLogf(char *formatStr, ...);
static void DemoCheckRc(IN OTX_HANDLE hDevice, IN OTX_RESULT otxRc);
static void DemoExitQuestion(void);

static OTX_INT32 DemoProcessUserCommand(char commandChar);

/*******************************************************************************
 Global Data
*******************************************************************************/
OTX_EVENT   g_hNotificationEvent = OTX_INVALID_EVENT_VALUE;
OTX_HANDLE  g_hEventQueue  = 0;
FILE       *g_pDemoLogFile = NULL;
FILE       *g_pFrameRxFile;
DemoEnvT    g_demoEnv;
OTX_BOOL    g_bHdlcVerbose = OTX_FALSE;
OTX_UINT32  g_coHdlcRxBytes[32];
OTX_UINT32  g_coHdlcRxFrames[32];
OTX_UINT32  g_coHdlcBadRxFrames[32];

OTX_BOOL    g_bSs7SimulationActive[32] = { 0 };
OTX_INT32   g_nSs7SimulationFramesReceived[32] = { 0 };

OtxHdlcSS7FisuS g_tFisu0;
OtxHdlcSS7FisuS g_tFisu1;
OtxHdlcSS7ShortLssuS g_tLssu;

OTX_UINT8  g_nPreviousFisu0[3];
OTX_UINT8  g_nPreviousFisu1[3];
OTX_UINT8  g_nCurrentFisu0[3];
OTX_UINT8  g_nCurrentFisu1[3];

OTX_UINT8  g_nPreviousLssu0[4];
OTX_UINT8  g_nPreviousLssu1[4];
OTX_UINT8  g_nCurrentLssu0[4];
OTX_UINT8  g_nCurrentLssu1[4];

OsaMutexS g_demoLogfCritSec;
OsaMutexS g_demoSendSs7DataCritSec;

struct {
  OTX_HANDLE hThorController0;
  OTX_HANDLE hQuadE1T1[4]; // Only used for Thor-8-PCI, Thor-8-PCI-Plus and Gimle-16-PCI-Plus
  OTX_HANDLE hVidarAsmController0;  // Not used for Thor-2-PCMCIA
  OTX_HANDLE hLi[16];
  OTX_HANDLE hDsp[4];
} g_hPhysDevs;

struct {
  OTX_HANDLE hPipe;
  OTX_HANDLE hHandle;

  OTX_INT32 coMsusSent;
  OTX_INT32 coFisusSent;

  OtxHdlcSS7FisuS fisu;

  OTX_INT32 busyCount;  // Should always be zero in this app

  OTX_INT32  nLi;
  OTX_INT32  nDsp;
} g_sqDspHdlcSender[32];

struct {
  OTX_HANDLE hPipe;
  OTX_HANDLE hHandle;

  OTX_UINT8  nPrevFsn;
  OTX_INT32  nSs7Errors;

  OTX_INT32  nLi;
  OTX_INT32  nDsp;
} g_sqDspHdlcReceiver[32];


struct {
  OtxE1ConfigOptionsS e1CfgData;
  OtxT1ConfigOptionsS t1CfgData;
} g_T1E1CfgData;


#if defined(_WIN32_WCE)
#define CLOCKS_PER_SEC 1
typedef int clock_t;
int clock() { return 0; }
#endif 
/*******************************************************************************
*******************************************************************************/


/*******************************************************************************

  @func

  Create the event queue to be used by the driver to inform the application
  about events, such as line status changes, or detected DMTF digits.

*******************************************************************************/
void DemoCreateEventQueue(void)
{
  OTX_RESULT result;

  // Create a notification Event
  result = OtxDrvCreateEvent(&g_hNotificationEvent); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);

  // Create Event Queue for driver notifications
  result = OtxDrvCreateEventQueue(128, g_hNotificationEvent, &g_hEventQueue); DemoCheckRc(g_hEventQueue, result);

  // Enable Event Queue
  result = OtxDrvEnable(g_hEventQueue, OTX_TASK_SYNC,0); DemoCheckRc(g_hEventQueue, result);
}



/*******************************************************************************

  @func
    Open the Physical Devices on the Thor board

  @comm
    Calls function <f OtxDrvCreatePhysicalDevice> returning a handle to a
    Device Object representing a physical device, which can be the
    entire board or an integrated circuit (chip) on the board. The number of
    physical devices is set per board and cannot change dynamically.

*******************************************************************************/
void DemoOpenPhysicalDevices(void)
{
  OTX_RESULT result;
  OTX_INT32  i;
  OTX_INT32  nQuadE1T1; 

  // Connect to the Physical Board Controller
  switch (g_demoEnv.nBoardType) {
  case OTX_DEVICE_THOR_PCI:
    DemoLogf("Opening Thor-2-PCI board\n");
    break;
  case OTX_DEVICE_THOR_PCMCIA:
    DemoLogf("Opening Thor-2-PCMCIA variant board\n");
    break;
  case OTX_DEVICE_THOR_2_PCI_PLUS:
    DemoLogf("Opening Thor-2-PCI-Plus board\n");
    break;
  case OTX_DEVICE_THOR_2_PCI_EXPRESS:
    DemoLogf("Opening Thor-2-PCI-Express board\n");
    break;
  case OTX_DEVICE_THOR_4_PCI_PLUS:
    DemoLogf("Opening Thor-4-PCI-Plus board\n");
    break;
  case OTX_DEVICE_THOR_8_PCI:
    DemoLogf("Opening Thor-8-PCI board\n");
    break;
  case OTX_DEVICE_THOR_8_PCI_PLUS:
    DemoLogf("Opening Thor-8-PCI-Plus board\n");
    break;
  case OTX_DEVICE_GIMLE_16_PCI_PLUS:
    DemoLogf("Opening Gimle-16-PCI-Plus board\n");
    break;
  case OTX_DEVICE_SLEIPNIR_1_PCI:
    DemoLogf("Opening Sleipnir-1-PCI board\n");
    break;
  default:
    DemoLogf("Board Type (%d) not supported by this demo program\n", g_demoEnv.nBoardType);
    exit(1);
    break;
  }
  result = OtxDrvOpenPhysicalDevice(0, g_demoEnv.nBoardType, g_demoEnv.boardNo, g_demoEnv.boardNo, g_hEventQueue, &g_hPhysDevs.hThorController0); DemoCheckRc(g_hPhysDevs.hThorController0, result);

  switch (g_demoEnv.nBoardType) {
  case OTX_DEVICE_SLEIPNIR_1_PCI:
  case OTX_DEVICE_THOR_PCMCIA:
  case OTX_DEVICE_THOR_PCI:
  case OTX_DEVICE_THOR_2_PCI_PLUS:
  case OTX_DEVICE_THOR_2_PCI_EXPRESS:
    // On these boards the E1T1 devices are hosted by the board itself. Get handles to T1/E1 line interface devices #0 and #1
    for (i = 0; i < g_demoEnv.coLi; i++) {
      DemoLogf("Open OTX_DEVICE_LI_T1E1 #%d\n", i);
      result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hThorController0, OTX_DEVICE_LI_T1E1, i, i, g_hEventQueue, &g_hPhysDevs.hLi[i]); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    }
    break;
  case OTX_DEVICE_THOR_4_PCI_PLUS:
    {
      // On the Thor-4-PCI-Plus board the E1T1 devices are hosted by a quad E1T1 device. 
      for (nQuadE1T1 = 0; nQuadE1T1 < 1; nQuadE1T1++) {
        DemoLogf("Open OTX_DEVICE_QUAD_T1E1 #%d\n", nQuadE1T1);
        result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hThorController0, OTX_DEVICE_QUAD_T1E1, nQuadE1T1, nQuadE1T1, g_hEventQueue, &g_hPhysDevs.hQuadE1T1[nQuadE1T1]); DemoCheckRc(g_hPhysDevs.hThorController0, result);
      }
      
      // Get handles to T1/E1 line interface devices 
      for (i = 0; i < g_demoEnv.coLi; i++) {
        nQuadE1T1 = i/4;
        DemoLogf("Open OTX_DEVICE_LI_T1E1 #%d (hosted by OTX_DEVICE_QUAD_T1E1 #%d)\n", i, nQuadE1T1);
        result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hQuadE1T1[nQuadE1T1], OTX_DEVICE_LI_T1E1, i%4, i, g_hEventQueue, &g_hPhysDevs.hLi[i]); DemoCheckRc(g_hPhysDevs.hQuadE1T1[nQuadE1T1], result);
      }
    }
    break;
  case OTX_DEVICE_THOR_8_PCI:
  case OTX_DEVICE_THOR_8_PCI_PLUS:
    {
      // On the Thor-8-PCI and Thor-8-PCI-Plus board the E1T1 devices are hosted by a quad E1T1 device. Open those two devices.
      for (nQuadE1T1 = 0; nQuadE1T1 < 2; nQuadE1T1++) {
        DemoLogf("Open OTX_DEVICE_QUAD_T1E1 #%d\n", nQuadE1T1);
        result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hThorController0, OTX_DEVICE_QUAD_T1E1, nQuadE1T1, nQuadE1T1, g_hEventQueue, &g_hPhysDevs.hQuadE1T1[nQuadE1T1]); DemoCheckRc(g_hPhysDevs.hThorController0, result);
      }
      
      // Get handles to T1/E1 line interface devices 
      for (i = 0; i < g_demoEnv.coLi; i++) {
        nQuadE1T1 = i/4;
        DemoLogf("Open OTX_DEVICE_LI_T1E1 #%d (hosted by OTX_DEVICE_QUAD_T1E1 #%d)\n", i, nQuadE1T1);
        result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hQuadE1T1[nQuadE1T1], OTX_DEVICE_LI_T1E1, i%4, i, g_hEventQueue, &g_hPhysDevs.hLi[i]); DemoCheckRc(g_hPhysDevs.hQuadE1T1[nQuadE1T1], result);
      }
    }
    break;
  case OTX_DEVICE_GIMLE_16_PCI_PLUS:
    {
      // On the Gimle-16-PCI-Plus board the E1T1 devices are hosted by 4 quad E1T1 devices. Open those four devices.
      for (nQuadE1T1 = 0; nQuadE1T1 < 4; nQuadE1T1++) {
        DemoLogf("Open OTX_DEVICE_QUAD_T1E1 #%d\n", nQuadE1T1);
        result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hThorController0, OTX_DEVICE_QUAD_T1E1, nQuadE1T1, nQuadE1T1, g_hEventQueue, &g_hPhysDevs.hQuadE1T1[nQuadE1T1]); DemoCheckRc(g_hPhysDevs.hThorController0, result);
      }
      
      // Get handles to T1/E1 line interface devices 
      for (i = 0; i < g_demoEnv.coLi; i++) {
        nQuadE1T1 = i/4;
        DemoLogf("Open OTX_DEVICE_LI_T1E1 #%d (hosted by OTX_DEVICE_QUAD_T1E1 #%d)\n", i, nQuadE1T1);
        result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hQuadE1T1[nQuadE1T1], OTX_DEVICE_LI_T1E1, i%4, i, g_hEventQueue, &g_hPhysDevs.hLi[i]); DemoCheckRc(g_hPhysDevs.hQuadE1T1[nQuadE1T1], result);
      }
    }
    break;
  default:
    DemoLogf("Board Type (%d) not supported by this demo program\n", g_demoEnv.nBoardType);
    exit(1);
    break;
  }

  // Open physical DSP devices
  switch (g_demoEnv.nBoardType) {
  case OTX_DEVICE_THOR_PCMCIA:
  case OTX_DEVICE_SLEIPNIR_1_PCI:
    // Get handles to the physcial DSP devices
    for (i = 0; i < g_demoEnv.coDsp; i++) {
      result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hThorController0, OTX_DEVICE_DSP, i, i, g_hEventQueue, &g_hPhysDevs.hDsp[i]); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    }
    break;
  case OTX_DEVICE_THOR_PCI:
  case OTX_DEVICE_THOR_2_PCI_PLUS: 
  case OTX_DEVICE_THOR_2_PCI_EXPRESS: 
  case OTX_DEVICE_THOR_4_PCI_PLUS: 
  case OTX_DEVICE_THOR_8_PCI: 
  case OTX_DEVICE_THOR_8_PCI_PLUS: 
  case OTX_DEVICE_GIMLE_16_PCI_PLUS:
    {
      OTX_UINT32   eDevType;
      OTX_ITERATOR itrChildTypes;
      OtxDrvIteratorBegin(&itrChildTypes);
      while (OTX_SUCCESS(OtxDrvEnumDeviceType(g_hPhysDevs.hThorController0, &itrChildTypes, (OTX_INT32 *)&eDevType))) {  
        OTX_BOOL bOpenAsm = OTX_FALSE;
        switch (eDevType) {
        case OTX_DEVICE_VIDAR_5x_ASM:
          DemoLogf("Opening Vidar-5x4-ASM board\n");
          bOpenAsm = OTX_TRUE;
          break;
        case OTX_DEVICE_VIDAR_5X4_ASM_CST:
          DemoLogf("Opening Vidar-5x4-ASM-CST board\n");
          bOpenAsm = OTX_TRUE;
          break;
        case OTX_DEVICE_VIDAR_5X4_ASM_V90:
          DemoLogf("Opening Vidar-5x4-ASM-V90 board\n");
          bOpenAsm = OTX_TRUE;
          break;
        case OTX_DEVICE_VIDAR_5X4_ASM_EX:
          DemoLogf("Opening Vidar-5x4-ASM-EX board\n");
          bOpenAsm = OTX_TRUE;
          break;
        case OTX_DEVICE_VIDAR_5X4_ASM_PRO:
          DemoLogf("Opening Vidar-5x4-ASM-PRO board\n");
          bOpenAsm = OTX_TRUE;
          break;
        case OTX_DEVICE_VIDAR_55x4_ASM:
          DemoLogf("Opening Vidar-55x4-ASM board\n");
          bOpenAsm = OTX_TRUE;

          // Verify that the system supports the Vidar-55x4-ASM board
          {
            char szCpldBuildNumbers[256];
            char szBrdBuildNumber[16];
            OTX_INT32 nBoardBuildNo;

            if (g_demoEnv.nBoardType == OTX_DEVICE_THOR_PCI) {
              
              //  Verify firmware revision
              result = OtxDrvGetAttributeValueSz(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_CPLD_BUILD, szCpldBuildNumbers, 256); DemoCheckRc(g_hPhysDevs.hThorController0, result);
              if (strstr(szCpldBuildNumbers, "OTXCDC: 0761") || strstr(szCpldBuildNumbers, "OTXCDC: 0762")) {
                DemoLogf("Error: The firmware on the Thor-2-PCI must be upgraded to firmware build 5\n");
                DemoLogf("       (OTXCDC 0764) or later to support the Vidar-55x4-ASM board.\n");
                DemoLogf("       Please download firmware update program from the Odin TeleSystems\n");
                DemoLogf("       web site (www.odints.com)\n");
                DemoLogf("       Also please verify that the HMA-1098-1-2.0 module is plugged into connector BJ3.\n");
                DemoExitQuestion();
              }
              
              // Verify board build number
              result = OtxDrvGetAttributeValueSz(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_BUILD, szBrdBuildNumber, 16); DemoCheckRc(g_hPhysDevs.hThorController0, result);
              nBoardBuildNo = strtol(strtok(szBrdBuildNumber, "."), NULL, 0);
              DemoLogf("Board Build Number: %d\n", nBoardBuildNo);
              if (nBoardBuildNo < 11) {
                DemoLogf("Error: The Thor-2-PCI board must be upgraded to build 11\n");
                DemoLogf("       or later to support the Vidar-55x4-ASM board.\n");
                DemoLogf("       Please contact Odin TeleSystems Support Team for upgrade details.\n");
                DemoExitQuestion();
              }
              
            } else if (g_demoEnv.nBoardType == OTX_DEVICE_THOR_8_PCI) {
              //  Verify firmware revision
              result = OtxDrvGetAttributeValueSz(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_CPLD_BUILD, szCpldBuildNumbers, 256); DemoCheckRc(g_hPhysDevs.hThorController0, result);
              if (strstr(szCpldBuildNumbers, "OTXCDC: 0771")) {
                DemoLogf("Error: The firmware on the Thor-8-PCI must be upgraded to firmware build 3\n");
                DemoLogf("       (OTXCDC 0772) or later to support the Vidar-55x4-ASM board.\n");
                DemoLogf("       Please download firmware update program from the Odin TeleSystems\n");
                DemoLogf("       web site (www.odints.com)\n");
                DemoLogf("       Also please verify that the HMA-1098-1-2.0 module is plugged into connector BJ3.\n");
                DemoExitQuestion();
              }
              
              // Verify board build number
              result = OtxDrvGetAttributeValueSz(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_BUILD, szBrdBuildNumber, 16); DemoCheckRc(g_hPhysDevs.hThorController0, result);
              nBoardBuildNo = strtol(szBrdBuildNumber, NULL, 0);
              DemoLogf("Board Build Number: %d\n", nBoardBuildNo);
              if (nBoardBuildNo < 5) {
                DemoLogf("Error: The Thor-8-PCI board must be upgraded to build 5\n");
                DemoLogf("       or later to support the Vidar-55x4-ASM board.\n");
                DemoLogf("       Please contact Odin TeleSystems Support Team for upgrade details.\n");
                DemoExitQuestion();
              }
              
            }
            
          }


          break;
        }
        if (bOpenAsm) {
          // Get a handle to to the ASM Daughter Board
          result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hThorController0, (OtxPhysicalDeviceTypeE) eDevType, 0, 0, g_hEventQueue, &g_hPhysDevs.hVidarAsmController0); DemoCheckRc(g_hPhysDevs.hThorController0, result);
        }
      }
      
      for (i = 0; i < g_demoEnv.coDsp; i++) {
        result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hVidarAsmController0, OTX_DEVICE_DSP, i, i, g_hEventQueue, &g_hPhysDevs.hDsp[i]); DemoCheckRc(g_hPhysDevs.hVidarAsmController0, result);
      }
    }
    break;
  default:
    DemoLogf("Board Type (%d) not supported by this demo program\n", g_demoEnv.nBoardType);
    exit(1);
    break;
  }
/*
  if (g_demoEnv.nBoardType == OTX_DEVICE_SLEIPNIR_1_PCI) {

    // we'll source the Falc from the DSP device, and source the DSP device from the FALC
    //  Falc <-- Burst Send (P2S)
    result = OtxDrvSetAttributeValueUint32(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_FALC_SOURCED, OTX_BRD_FALC_SOURCED_DSP); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    //  Burst Recv (S2P) <-- Falc
    result = OtxDrvSetAttributeValueUint32(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_DSP_SOURCED, OTX_BRD_DSP_SOURCED_FALC); DemoCheckRc(g_hPhysDevs.hThorController0, result);
  }
*/

	// Load DSP Program
  for (i = 0; i < g_demoEnv.coDsp; i++) {
    switch (g_demoEnv.eProgramModule) {
    case OTX_DSP_SPM_OTXSPM1:
      DemoLogf("DSP#%d: Loading DSP program OTX_DSP_SPM_OTXSPM1\n", i);
      break;
    case OTX_DSP_SPM_OTXSPM2:
      DemoLogf("DSP#%d: Loading DSP program OTX_DSP_SPM_OTXSPM2\n", i);
      break;
    case OTX_DSP_SPM_OTXHDLC:
      DemoLogf("DSP#%d: Loading DSP program OTX_DSP_SPM_OTXHDLC\n", i);
      break;
    case OTX_DSP_SPM_OTXHDLCEX:
      DemoLogf("DSP#%d: Loading DSP program OTX_DSP_SPM_OTXHDLCEX\n", i);
      break;
    default:
      DemoLogf("Unsupported DSP program module (%d). Exiting.\n", g_demoEnv.eProgramModule);
      OsaConsoleExit();
      OtxDrvReleaseLib();
      exit(1);
    }
#if 1
    result = OtxDspRunSpm(g_hPhysDevs.hDsp[i], g_demoEnv.eProgramModule); DemoCheckRc(g_hPhysDevs.hDsp[i], result);
#else
    DemoLogf("Loading from file\n");
    //result = OtxDspRunProgram(g_hPhysDevs.hDsp[i], "OtxSpm2.out", "_c_int00"); DemoCheckRc(g_hPhysDevs.hDsp[i], result);
    //result = OtxDspRunProgram(g_hPhysDevs.hDsp[i], "OtxSpm2c5410a.out", "_c_int00"); DemoCheckRc(g_hPhysDevs.hDsp[i], result);
    //result = OtxDspRunProgram(g_hPhysDevs.hDsp[i], "OtxHdlc5410a.out", "_c_int00"); DemoCheckRc(g_hPhysDevs.hDsp[i], result);
    result = OtxDspRunProgram(g_hPhysDevs.hDsp[i], "OtxHdlcEx5416.out", "_c_int00"); DemoCheckRc(g_hPhysDevs.hDsp[i], result);
#endif
  }

  if (g_demoEnv.bUseDspDebugger) {
    DemoLogf("Please load and run DSP program %s to DSP#%d-%d in your DSP Debugger\n", "OtxHdlc.out", 0, g_demoEnv.coDsp-1);
    DemoLogf("Press any key when all the DSP programs are running...\n");
    OsaConsoleGetCh();
    
    // At this point we need to tell the host to restart the programs 
    // (to issue the initial IO commands to the DSP again)
    for (i = 0; i < g_demoEnv.coDsp; i++) {
      DemoLogf("Restarting program on DSP#%d\n", i);
      result = OtxDspRestartProgram(g_hPhysDevs.hDsp[i]); DemoCheckRc(g_hPhysDevs.hDsp[i], result);
    }
    // At this point the DSP should be in the same state as they were right after OtxDspRunProgram() was called
  }


}



/*******************************************************************************

  @func
    Configure E1/T1 Devices

  @comm The clock source of the board(s) must set by a call 
  to <f OtxBrdSetClocks> prior to calling this function.

*******************************************************************************/
void DemoConfigureE1T1Devices(void)
{
  OTX_RESULT result;
  OTX_INT32  i;

  // Initialize the T1/E1 Line Interface Devices

  // Set default configuration data
  DemoInitT1E1ConfigData( &(g_T1E1CfgData.e1CfgData), &(g_T1E1CfgData.t1CfgData));

  // Configure Transceivers
  for (i = 0; i < g_demoEnv.coLi; i++) {

    // Set termination for the T1/E1 Front End
    result = OtxT1E1SetLineTermination(g_hPhysDevs.hLi[i], OTX_T1E1_TERM_100_OHM); DemoCheckRc(g_hPhysDevs.hLi[i], result);

    if (g_demoEnv.eT1E1Mode == OTX_T1E1_MODE_T1) {
      // T1 mode
      result = OtxT1SetConfiguration(g_hPhysDevs.hLi[i], &(g_T1E1CfgData.t1CfgData));  DemoCheckRc(g_hPhysDevs.hLi[i], result);
    } else {
      // E1 mode
      result = OtxE1SetConfiguration(g_hPhysDevs.hLi[i], &(g_T1E1CfgData.e1CfgData));  DemoCheckRc(g_hPhysDevs.hLi[i], result);
    }
    result = OtxT1E1SetLiMode(g_hPhysDevs.hLi[i], g_demoEnv.eT1E1Mode); DemoCheckRc(g_hPhysDevs.hLi[i], result);
  }

}

/*******************************************************************************

  @func
    Enable Configured Physical Devices

*******************************************************************************/
void DemoEnablePhysicalDevices(void)
{
  OTX_RESULT result;
  OTX_INT32  i;
  OTX_INT32  nQuadE1T1; 

  result = OtxDrvEnable(g_hPhysDevs.hThorController0,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hThorController0, result);

  if (g_hPhysDevs.hVidarAsmController0) {
    result = OtxDrvEnable(g_hPhysDevs.hVidarAsmController0,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hVidarAsmController0, result);
  }

  // If board has Quad E1T1 devices then enable them
  switch (g_demoEnv.nBoardType) {
  case OTX_DEVICE_THOR_4_PCI_PLUS:
    {      
      // Enable Quad E1T1 devices
      for (nQuadE1T1 = 0; nQuadE1T1 < 1; nQuadE1T1++) {
        DemoLogf("Enable OTX_DEVICE_QUAD_T1E1 #%d device\n", nQuadE1T1);
        result = OtxDrvEnable(g_hPhysDevs.hQuadE1T1[nQuadE1T1], OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hQuadE1T1[nQuadE1T1], result);
      }
    }
    break;
  case OTX_DEVICE_THOR_8_PCI:
  case OTX_DEVICE_THOR_8_PCI_PLUS:
    {      
      // Enable Quad E1T1 devices
      for (nQuadE1T1 = 0; nQuadE1T1 < 2; nQuadE1T1++) {
        DemoLogf("Enable OTX_DEVICE_QUAD_T1E1 #%d device\n", nQuadE1T1);
        result = OtxDrvEnable(g_hPhysDevs.hQuadE1T1[nQuadE1T1], OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hQuadE1T1[nQuadE1T1], result);
      }
    }
    break;
  case OTX_DEVICE_GIMLE_16_PCI_PLUS:
    {
      // Enable Quad E1T1 devices
      for (nQuadE1T1 = 0; nQuadE1T1 < 4; nQuadE1T1++) {
        DemoLogf("Enable OTX_DEVICE_QUAD_T1E1 #%d device\n", nQuadE1T1);
        result = OtxDrvEnable(g_hPhysDevs.hQuadE1T1[nQuadE1T1], OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hQuadE1T1[nQuadE1T1], result);
      }
    }
    break;
  default:
    // Nothing
    break;
  }

  for (i = 0; i < g_demoEnv.coLi; i++) {
    result = OtxDrvEnable(g_hPhysDevs.hLi[i], OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hLi[i], result);
  }

  for (i = 0; i < g_demoEnv.coDsp; i++) {
    result = OtxDrvEnable(g_hPhysDevs.hDsp[i], OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hDsp[i], result);
  }
}



/*******************************************************************************

  @func
    Create the needed Logical Devices

  @comm
    The logical devices are differenciated from the phyiscal devices by the fact
    the number of logical devices is not set. More of them can be requested
    by the user and allocated by the system if needed.

*******************************************************************************/
void DemoCreateLogicalDevices(
  void
)
{
  OTX_RESULT result;
  OTX_INT32  i;
#if 0
  OTX_UINT32 nIrpQueueLimit;
#endif


  for (i = 0; i < g_demoEnv.coLi*g_demoEnv.coHdlcPerLi; i++) {
    if (g_demoEnv.bUseSenders) {
      result = OtxDrvCreateLogicalDevice(g_hPhysDevs.hDsp[i%g_demoEnv.coDsp], OTX_LDEVICE_HDLC_SENDER,   g_hEventQueue, i, &g_sqDspHdlcSender[i].hHandle); DemoCheckRc(g_hPhysDevs.hDsp[i%g_demoEnv.coDsp], result);
    }
    if (g_demoEnv.bUseReceivers) {
      result = OtxDrvCreateLogicalDevice(g_hPhysDevs.hDsp[(i+2)%g_demoEnv.coDsp], OTX_LDEVICE_HDLC_RECEIVER, g_hEventQueue, i, &g_sqDspHdlcReceiver[i].hHandle); DemoCheckRc(g_hPhysDevs.hDsp[(i+2)%g_demoEnv.coDsp], result);
    }

#if 0
    {
      OTX_INT32 nIrpQueueLimit = 100; 
      
      DemoLogf("HdlcSender[%d]: Setting nIrpQueueLimit = %d\n", i, nIrpQueueLimit);
      result = OtxDrvSetAttributeValueUint32(g_sqDspHdlcSender[i].hHandle, OTX_ATTR_DEVICE_IRP_QUEUE_LIMIT, nIrpQueueLimit); DemoCheckRc(g_sqDspHdlcSender[i].hHandle, result);
    }
#endif
  }
  
}



/*******************************************************************************

  @func
    Create the needed Pipe Devices and connect the physical and logical devices
    to pipes.

  @comm
    A Pipe Device is a logical entity representing a serial bit-stream (a subset of a
    Highway) with a data rate between 8 kbit/s and 2.048Mbit/s. A pipe is a characterized
    bu its capacity (in 8 kbit/s increments). The user can create a pipe with
    the driver function <f OtxDrvCreatePipe>. A pipe is connected to physical or
    logical devices and mapped to specific time-slots with the driver function
    <f OtxDrvConnectPipe>.

*******************************************************************************/
void DemoConnectDevices(
  void
)
{
  OTX_INT32 i;
  OTX_RESULT result;
  OTX_UINT8 sqTimeSlotMask[32];
  OTX_INT32 nLi;

    // 64kbps or 56kbps HDLC capacity
    OTX_INT32 j;
    OTX_INT32 ts;
    OTX_INT32 nCapacity;
    
    if (g_demoEnv.b56kbpsChannel) {
      DemoLogf("Configuring one 56kbps HDLC channel per Li\n");
      nCapacity = 56;
    } else {
      DemoLogf("Configuring one 64kbps HDLC channel per Li\n");
      nCapacity = 64;
    }

    i = 0;
    for (nLi = 0; nLi < g_demoEnv.coLi; nLi++) {
      
      if (g_demoEnv.eT1E1Mode == OTX_T1E1_MODE_T1) {
        // T1 mode
        ts = 24; 
      } else {
        // E1 mode
        ts = 16; 
      }
      
      for (j = 0; j < g_demoEnv.coHdlcPerLi; j++, i++, ts--) {
        
        // Connect pipes to the E1/T1 device
        memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
        if (g_demoEnv.b56kbpsChannel) {
          sqTimeSlotMask[ts] = 0xFE; // Timeslot on the T1/E1 span
        } else {
          sqTimeSlotMask[ts] = 0xFF; // Timeslot on the T1/E1 span
        }
        
        if (g_demoEnv.bUseReceivers) {
          result = OtxDrvCreatePipe(nCapacity, &g_sqDspHdlcReceiver[i].hPipe); DemoCheckRc(g_sqDspHdlcReceiver[i].hPipe, result);
          
          g_sqDspHdlcReceiver[i].nLi = nLi;
          result = OtxDrvConnectPipe(g_sqDspHdlcReceiver[i].hPipe, OTX_PIPE_INPUT,  g_hPhysDevs.hLi[nLi],  -1, sqTimeSlotMask, 32); DemoCheckRc(g_sqDspHdlcReceiver[i].hPipe, result);
          
          g_sqDspHdlcReceiver[i].nDsp = (i+2)%g_demoEnv.coDsp;
          
          if ((g_demoEnv.nBoardType == OTX_DEVICE_THOR_PCMCIA) || (g_demoEnv.nBoardType == OTX_DEVICE_SLEIPNIR_1_PCI) || g_demoEnv.b56kbpsChannel) {
            // Connect pipes to the DSP device (Fixed timeslots since the PCMCIA board does not have a time-space switch)
            result = OtxDrvConnectPipe(g_sqDspHdlcReceiver[i].hPipe, OTX_PIPE_OUTPUT, g_hPhysDevs.hDsp[g_sqDspHdlcReceiver[i].nDsp], -1, sqTimeSlotMask, 32); DemoCheckRc(g_sqDspHdlcReceiver[i].hPipe, result);
          } else {
            // Connect pipes to the DSP device (Arbitrary timeslot on the DSP highway)
            result = OtxDrvConnectPipe(g_sqDspHdlcReceiver[i].hPipe, OTX_PIPE_OUTPUT, g_hPhysDevs.hDsp[g_sqDspHdlcReceiver[i].nDsp], -1, NULL, 0); DemoCheckRc(g_sqDspHdlcReceiver[i].hPipe, result);
          }
          
          // Create the following Connections in receive direction:
          //  Li  ------> HDLC Receiver
          result = OtxDrvConnectLogicalDevice(g_sqDspHdlcReceiver[i].hPipe, -1, g_sqDspHdlcReceiver[i].hHandle,       -1); DemoCheckRc(g_sqDspHdlcReceiver[i].hPipe, result);
          
          DemoLogf("Hdlc Receiver[%d] connected (Dsp#%d, Li#%d) \n", i, g_sqDspHdlcReceiver[i].nDsp, g_sqDspHdlcReceiver[i].nLi);
        }
        
        
        if (g_demoEnv.bUseSenders) {
          result = OtxDrvCreatePipe(nCapacity, &g_sqDspHdlcSender[i].hPipe); DemoCheckRc(g_sqDspHdlcSender[i].hPipe, result);
          
          g_sqDspHdlcSender[i].nLi = nLi;
          result = OtxDrvConnectPipe(g_sqDspHdlcSender[i].hPipe,   OTX_PIPE_OUTPUT, g_hPhysDevs.hLi[nLi],  -1, sqTimeSlotMask, 32); DemoCheckRc(g_sqDspHdlcSender[i].hPipe, result);
          
          g_sqDspHdlcSender[i].nDsp = i%g_demoEnv.coDsp; 
          
          if ((g_demoEnv.nBoardType == OTX_DEVICE_THOR_PCMCIA) || (g_demoEnv.nBoardType == OTX_DEVICE_SLEIPNIR_1_PCI) || g_demoEnv.b56kbpsChannel) {
            // Connect pipes to the DSP device (Fixed timeslots since the PCMCIA board does not have a time-space switch)
            result = OtxDrvConnectPipe(g_sqDspHdlcSender[i].hPipe,   OTX_PIPE_INPUT,  g_hPhysDevs.hDsp[g_sqDspHdlcSender[i].nDsp], -1, sqTimeSlotMask, 32); DemoCheckRc(g_sqDspHdlcSender[i].hPipe, result);
          } else {
            // Connect pipes to the DSP device (Arbitrary timeslot on the DSP highway)
            result = OtxDrvConnectPipe(g_sqDspHdlcSender[i].hPipe,   OTX_PIPE_INPUT,  g_hPhysDevs.hDsp[g_sqDspHdlcSender[i].nDsp], -1, NULL, 0); DemoCheckRc(g_sqDspHdlcSender[i].hPipe, result);
          }
          
          // Make the following connections in transmit direction:
          // HDLC Sender   -----> Li
          result = OtxDrvConnectLogicalDevice(g_sqDspHdlcSender[i].hHandle, -1, g_sqDspHdlcSender[i].hPipe, -1); DemoCheckRc(g_sqDspHdlcSender[i].hHandle, result);
          
          DemoLogf("Hdlc Sender[%d]   connected (Dsp#%d, Li#%d) \n", i, g_sqDspHdlcSender[i].nDsp, g_sqDspHdlcSender[i].nLi);
        }        
      }
    }
    
      


}


/*******************************************************************************

  @func
    Initialize the logical device. This function must be called after
    the devices are created and connected.

*******************************************************************************/
void DemoInitializeLogicalDevices(
  void
)
{
  OTX_RESULT result;
  OTX_INT32  i;

  for (i = 0; i < g_demoEnv.coLi*g_demoEnv.coHdlcPerLi; i++) {
    
    if (g_demoEnv.bUseSenders) {
    /*    {
    // Specify whether the HDLC Sender device should send us the OTX_HDLC_SENDER_EC_FRAME_SENT event
    OTX_BOOL    bHdlcFrameSentEvents = OTX_TRUE;
    
      DemoLogf("HDLC Sender[%d] Setting OTX_ATTR_HDLC_SENDER_GENERATE_FRAME_SENT_EVENT=%s\n", i, (bHdlcFrameSentEvents) ? "TRUE": "FALSE");
      result = OtxDrvSetAttributeValueBool(g_sqDspHdlcSender[i].hHandle, OTX_ATTR_HDLC_SENDER_GENERATE_FRAME_SENT_EVENT, bHdlcFrameSentEvents); DemoCheckRc(g_sqDspHdlcSender[i].hHandle, result);
      }
      */
      
#if 0
      // Reduce the IRP queue to 1 for the HDLC Sender 
      // (which is the number of HDLC frames that the driver will 
      // buffer before returning OTX_E_BUSY)
      // The default IRP queue size is 10 frames.
      result = OtxDrvSetAttributeValueUint32(
        g_sqDspHdlcSender[i].hHandle, 
        OTX_ATTR_DEVICE_IRP_QUEUE_LIMIT, 
        1); 
      DemoCheckRc(g_sqDspHdlcSender[i].hHandle, result);
#endif
      
      
      // Initialize the Hdlc Sender 
      result = OtxHdlcSenderInit(
        g_sqDspHdlcSender[i].hHandle, // Handle to logical HDLC Filter Device
        OTX_HDLC_FFT_FLAGS,        // Fill character(s) to be send between HDLC frames
        //OTX_HDLC_FFT_ALL_ONES,        // Fill character(s) to be send between HDLC frames
        1,                         // Minimum No of interframe time characters between 2 HDLC frames (1=shared flags, 2=non-shared) Min value is 1. Max is 1024.
        OTX_HDLC_CRC16,            // Cyclic Redundancy Check (CRC) mode: 16-bit, 32-bit, or none.
        g_demoEnv.bHdlcBitInversion, // If OTX_TRUE then all bits are inverted when transmitted and before received
        OTX_TRUE,                  // If OTX_TRUE the HDLC device will append the CRC to the frame to be transmitted. If OTX_FALSE, the CRC is generated with software
        OTX_HDLC_TR_NA             // HDLC Transmission Rate for V.110 and X.30 modes
        ); DemoCheckRc(g_sqDspHdlcSender[i].hHandle, result);          
    }
        
    if (g_demoEnv.bUseReceivers) {
      
      // Initialize the Hdlc Receiver 
      result = OtxHdlcReceiverInit(
        g_sqDspHdlcReceiver[i].hHandle, // Handle to logical HDLC Filter Device
        OTX_HDLC_FFT_FLAGS,        // Fill character(s) to be send between HDLC frames
        //OTX_HDLC_FFT_ALL_ONES,        // Fill character(s) to be send between HDLC frames
        1,                         // Minimum No of interframe time characters between 2 HDLC frames (1=shared flags, 2=non-shared) Min value is 1. Max is 1024.
        OTX_HDLC_CRC16,            // Cyclic Redundancy Check (CRC) mode: 16-bit, 32-bit, or none.
        g_demoEnv.bHdlcBitInversion, // If OTX_TRUE then all bits are inverted when transmitted and before received
        OTX_TRUE,                  // If OTX_TRUE the HDLC device will remove the CRC from the received frame, if OTX_FALSE the CRC is passed on to the receiving software
        OTX_HDLC_TR_NA,            // HDLC Transmission Rate for V.110 and X.30 modes
        OTX_TRUE                  // If OTX_TRUE the HDLC device will generate an event for every received frame, if OTX_FALSE then the HDLC device will only generate an event when one or more frames have been received
        ); DemoCheckRc(g_sqDspHdlcReceiver[i].hHandle, result);          
      
      if (g_demoEnv.bSs7FisuFiltering) {
        DemoLogf("Setting FISU and LSSU filtering for device %d\n", i);
        result = OtxHdlcSS7SetFilter(g_sqDspHdlcReceiver[i].hHandle, 
          OTX_HDLC_SS7_FILTER_FISU | 
          OTX_HDLC_SS7_FILTER_SHORT_LSSU | 
          OTX_HDLC_SS7_FILTER_LONG_LSSU); 
        DemoCheckRc(g_sqDspHdlcReceiver[i].hHandle, result);
      }
    }
    
  }
}



/*******************************************************************************

  @func
    Enable Configured Logical Devices

*******************************************************************************/
void DemoEnableLogicalDevices(void)
{
  OTX_RESULT result;
  OTX_INT32  i;

  if (g_demoEnv.bUseSenders) {
    for (i = 0; i < g_demoEnv.coLi*g_demoEnv.coHdlcPerLi; i++) {
      //DemoLogf("** i=%d Enabling hSpm1Sender[%d] \n", i); OsaConsoleOsaConsoleGetCh();
      result = OtxDrvEnable(g_sqDspHdlcSender[i].hHandle,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_sqDspHdlcSender[i].hHandle, result);
    }
  }

  if (g_demoEnv.bUseReceivers) {
    for (i = 0; i < g_demoEnv.coLi*g_demoEnv.coHdlcPerLi; i++) {
      //DemoLogf("** i=%d Enabling hSpm1Receiver[%d] \n", i); OsaConsoleOsaConsoleGetCh();
      result = OtxDrvEnable(g_sqDspHdlcReceiver[i].hHandle,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_sqDspHdlcReceiver[i].hHandle, result);
    }
  }
  
}


/*******************************************************************************

  @func
    Initialize the T1/E1 Configuration Data Structures passed to the T1/E1
    Line Interface physical devices

*******************************************************************************/
void DemoInitT1E1ConfigData(
  OUT OtxE1ConfigOptionsS *pE1CfgData,
  OUT OtxT1ConfigOptionsS *pT1CfgData
)
{
  // Initialize E1 Configuration Data
  pE1CfgData->eTxLineCode                = g_demoEnv.eE1LineCode;
  pE1CfgData->eRxLineCode                = g_demoEnv.eE1LineCode;
  pE1CfgData->eTxFrameFormat             = g_demoEnv.eE1FrameFormat;
  pE1CfgData->eRxFrameFormat             = g_demoEnv.eE1FrameFormat;
#if 1
  pE1CfgData->eAisDetection              = OTX_E1_AIS_ETS300233;
#else
  pE1CfgData->eAisDetection              = OTX_E1_AIS_G775;
#endif
  pE1CfgData->bfSiBits                   = 0x3;
  pE1CfgData->bfSaBits                   = 0x1F;
  pE1CfgData->bExtendedHDB3errorDetect   = OTX_FALSE;
  pE1CfgData->bAutoRegainMultiframe      = OTX_TRUE;
  pE1CfgData->bAutoResync                = OTX_TRUE;
  pE1CfgData->bAutoCrcBitInversion       = OTX_FALSE;
  pE1CfgData->bAutoRemoteAlarm           = OTX_FALSE;
  pE1CfgData->nLosSensitivity            = 10;  // 10 = LOS detection after 176 consecutive zeros which fulfills G.775
  pE1CfgData->nLosRecovery               = 21;  // 21 = LOS recovery after 22 ones in the LosRecovery interval which fulfills G.775
  pE1CfgData->nLineLength                = 10;
  pE1CfgData->bHighTxPower               = OTX_FALSE;
  pE1CfgData->bReceiveEqualizer          = OTX_FALSE;

  // Initialize T1 Configuration Data
  pT1CfgData->eSignalingMode             = OTX_T1_SM_CCS;
  pT1CfgData->eTxLineCode                = g_demoEnv.eT1LineCode;
  pT1CfgData->eRxLineCode                = g_demoEnv.eT1LineCode;
  pT1CfgData->eFrameFormat               = g_demoEnv.eT1FrameFormat;
  pT1CfgData->bCRC6Enable                = OTX_TRUE;
  pT1CfgData->eTxRemoteAlarmFormat       = OTX_T1_RA_YELLOW_A;
  pT1CfgData->eRxRemoteAlarmFormat       = OTX_T1_RA_YELLOW_A;
  pT1CfgData->bAutoResync                = OTX_TRUE;
  pT1CfgData->bAutoCrcBitInversion       = OTX_FALSE;
  pT1CfgData->eLfaSensitivity            = OTX_T1_LFA_2_OUT_OF_4;
  pT1CfgData->bAutoRemoteAlarm           = OTX_FALSE;
  pT1CfgData->nLosSensitivity            = 10;  // 10 = LOS detection after 176 consecutive zeros which fulfills G.775, Telcordia (Bellcore) and AT&T
  pT1CfgData->nLosRecovery               = 21;  // 21 = LOS recovery after 22 ones in the LosRecovery interval which fulfills G.775, Telcordia (Bellcore) and AT&T
  pT1CfgData->nLineLength                = 10;
  pT1CfgData->bHighTxPower               = OTX_FALSE;
  pT1CfgData->bReceiveEqualizer          = OTX_TRUE;
  pT1CfgData->bfClearChannels            = 0x000000;
}



/*******************************************************************************
  @func
    Prints a timestamp in a MM-DD hh:mm:ss:mmm format, where

    MM  = month
    DD  = day (of month)
    hh  = hour of day
    mm  = minutes of day
    ss  = seconds of day
    mmm = milliseconds

*******************************************************************************/
void DemoPrintTimeStamp(
  OTX_DATETIME nTimeStamp
)
{
  OTX_DATETIME nTimeStampMs;
  OTX_DATETIME nTimeStampSec;
  struct Osa_tm *ptCurrentDayAndTime;
  Osa_time_t nCurrentTimeSecs;
  
  // Subtract number of 100ns ticks between Jan 1, 1970 and Jan 1, 1600
  
  #if defined(__linux__)
    nTimeStamp.nUint64 = nTimeStamp.nUint64 - 116444736000000000LL;  
  #elif defined(WIN32)
    nTimeStamp.nUint64 = nTimeStamp.nUint64 - 116444736000000000;  
  #endif

  
  // Convert to seconds since Jan 1, 1970 (so we can use this time with OsaStdLocalTime())
  nTimeStampSec.nUint64 = nTimeStamp.nUint64 / 10000000;
  
  // Calculate number of milliseconds (since number of seconds is not a very accurate way to report timestamp)
  nTimeStampMs.nUint64 = nTimeStamp.nUint64 / 10000 - (nTimeStampSec.nUint64 * 1000);
  
  nCurrentTimeSecs = nTimeStampSec.sUint32.nLow;

  ptCurrentDayAndTime = OsaStdLocalTime(&nCurrentTimeSecs);
  
  if (ptCurrentDayAndTime) {
    
    DemoLogf("%.2d-%.2d %.2d:%.2d:%.2d.%.3d ", 
      ptCurrentDayAndTime->tm_mon + 1,
      ptCurrentDayAndTime->tm_mday,
      ptCurrentDayAndTime->tm_hour,
      ptCurrentDayAndTime->tm_min, 
      ptCurrentDayAndTime->tm_sec,
      nTimeStampMs.sUint32.nLow
      );
  } else {
    //DemoLogf("Error: Invalid timestamp (before Jan 1, 1970)\n");
  }
  
}


/*******************************************************************************
  @func
    Prints a timestamp in a MM-DD hh:mm:ss:mmm format, where

    MM  = month
    DD  = day (of month)
    hh  = hour of day
    mm  = minutes of day
    ss  = seconds of day
    mmm = milliseconds

*******************************************************************************/
void DemoPrintTimeStamp2(
  OTX_DATETIME nTimeStamp
)
{
  OTX_DATETIME nTimeStampMs;
  OTX_DATETIME nTimeStampSec;
  struct Osa_tm *ptCurrentDayAndTime;
  Osa_time_t nCurrentTimeSecs;
  
  // Subtract number of 100ns ticks between Jan 1, 1970 and Jan 1, 1600
  
  #if defined(__linux__)
    nTimeStamp.nUint64 = nTimeStamp.nUint64 - 116444736000000000LL;  
  #elif defined(WIN32)
    nTimeStamp.nUint64 = nTimeStamp.nUint64 - 116444736000000000;  
  #endif

  
  // Convert to seconds since Jan 1, 1970 (so we can use this time with OsaStdLocalTime())
  nTimeStampSec.nUint64 = nTimeStamp.nUint64 / 10000000;
  
  // Calculate number of milliseconds (since number of seconds is not a very accurate way to report timestamp)
  nTimeStampMs.nUint64 = nTimeStamp.nUint64 / 10000 - (nTimeStampSec.nUint64 * 1000);
  
  nCurrentTimeSecs = nTimeStampSec.sUint32.nLow;

  ptCurrentDayAndTime = OsaStdLocalTime(&nCurrentTimeSecs);
  
  if (ptCurrentDayAndTime) {
    
    fprintf(g_pFrameRxFile, "%.2d-%.2d %.2d:%.2d:%.2d.%.3d ", 
      ptCurrentDayAndTime->tm_mon + 1,
      ptCurrentDayAndTime->tm_mday,
      ptCurrentDayAndTime->tm_hour,
      ptCurrentDayAndTime->tm_min, 
      ptCurrentDayAndTime->tm_sec,
      nTimeStampMs.sUint32.nLow
      );
  } else {
    //DemoLogf("Error: Invalid timestamp (before Jan 1, 1970)\n");
  }
  
}




/*******************************************************************************
  @func
    Idle Function that check for Events from the OTX Driver
*******************************************************************************/
void  *DemoIdleFunction(
  void *lpParameter   // thread data
)
{
  OtxEventDataS eventData;
  OTX_RESULT result;


  while (1) {
    OtxDrvWaitForSingleEvent(g_hNotificationEvent, OTX_TIME_INFINITY);
    //DemoLogf("Event signalled\n");
    do {
      result = OtxDrvGetEventData(g_hEventQueue, &eventData); DemoCheckRc(g_hEventQueue, result);

      if (OTX_RESULT_CODE(result) == OTX_S_OK) {
        // We have a driver event
        // Printf the event data.
        // In a real application branch based on the data
        DemoPrintEvent(&eventData);

      }
    } while (OTX_RESULT_CODE(result) == OTX_S_OK);
  }

  return 0;


}



/*******************************************************************************
  @func
    Prints a Driver Event. If HDLC Data, reads the data.
*******************************************************************************/
void DemoPrintEvent(
  OtxEventDataS *pEventData
)
{
  OTX_RESULT otxRc;
  OTX_UINT32 i;
  char msgStr[256];

#if 0
  if (g_bHdlcVerbose) {
    DemoPrintTimeStamp(pEventData->m_nTimeStamp);
  }

  switch (pEventData->m_eDeviceType) {
  case OTX_LDEVICE_HDLC_RECEIVER:
    if (g_bHdlcVerbose) {
      DemoPrintTimeStamp(pEventData->m_nTimeStamp);
    }
    break;
  default:
    DemoPrintTimeStamp(pEventData->m_nTimeStamp);
    break;
  }
#endif

  switch (pEventData->m_eDeviceType) {
    case OTX_DEVICE_LI_T1E1:
    // Event from Li0 or Li1
    otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
    if (OTX_SUCCESS(otxRc)) {
      if (pEventData->m_nParam) {
        DemoLogf("E1/T1[%d] Event: %s (%d)\n", pEventData->m_nSourceId, msgStr, pEventData->m_nParam);
      } else {
        DemoLogf("E1/T1[%d] Event: %s\n", pEventData->m_nSourceId, msgStr);
      }
    } else {
      DemoLogf("E1/T1[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
    }
    break;

  case OTX_DEVICE_DSP:
    // Non-logical device related event from the DSPs
    otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
    if (OTX_SUCCESS(otxRc)) {
       DemoLogf("DSP[%d] Event: %s (m_nParam=0x%X, m_nRequestId=0x%X)\n", pEventData->m_nSourceId, msgStr, pEventData->m_nParam, pEventData->m_nRequestId);

    //	DemoLogf("DSP[%d] Event: %s (m_nParam=%d, m_nRequestId=%d)\n", pEventData->m_nSourceId, msgStr, pEventData->m_nParam, pEventData->m_nRequestId);
    } else {
      DemoLogf("DSP[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
    }
    break;

  case OTX_LDEVICE_TONE_DTMF_DIALER:
    otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
    if (OTX_SUCCESS(otxRc)) {
      DemoLogf("DtmfDialler[%d] Event: %s \n", pEventData->m_nSourceId, msgStr);
    } else {
      DemoLogf("DtmfDialler[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
    }
    break;

  case OTX_LDEVICE_TONE_DTMF_DETECTOR:
    otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
    if (OTX_SUCCESS(otxRc)) {
      // Event from Dtmf Detector
      if (pEventData->m_nCode == OTX_DTMF_DETECTOR_EC_DIGIT) {
        DemoLogf("DtmfDetector[%d] detected DTMF digit %d \n", pEventData->m_nSourceId, pEventData->m_nParam);
      } else {
        // Unexpected event from this device
        DemoLogf("DtmfDetector[%d] Unexpected Event: %s \n", pEventData->m_nSourceId, msgStr);
      }
    } else {
      DemoLogf("DtmfDetector[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
    }
    break;

  case OTX_LDEVICE_HDLC_RECEIVER:
    // Event from HLDC Receiver
    if (pEventData->m_nCode == OTX_HDLC_RECEIVER_EC_COMPLETE_FRAME) {
      OTX_UINT8  buf[DEMO_HDLC_RX_BUFFER_SIZE];
      OTX_UINT32 nFrameLen;
      OTX_UINT32 nFrameSeq;
      OtxHdlcFrameEndCodeE nFrameEndCode;
      OTX_DATETIME nTimeStamp;
      OTX_RESULT result;
      OtxHdlcSS7FisuS *pFisu;
      OTX_INT32 nDev; 
      
      pFisu = 0; // Initialize to remove build warnings 

      otxRc = OtxHdlcReadData(pEventData->m_hDevice, buf, DEMO_HDLC_RX_BUFFER_SIZE, &nFrameLen, &nFrameSeq, &nFrameEndCode, &nTimeStamp);
      while (OTX_RESULT_CODE(otxRc) == OTX_S_OK) {

        // Log the received frames to a separate file
        DemoPrintTimeStamp2(pEventData->m_nTimeStamp);
        fprintf(g_pFrameRxFile, " %d, %d, %d, %d, ", pEventData->m_nSourceId, nFrameLen, nFrameEndCode, nFrameSeq);
        for (i = 0; i < nFrameLen && i < DEMO_HDLC_RX_BUFFER_SIZE; i++) {
          fprintf(g_pFrameRxFile, "%.2X ", buf[i]);
        }
        fprintf(g_pFrameRxFile, "\n");

        if (pEventData->m_nSourceId < 32) {
          g_coHdlcRxFrames[pEventData->m_nSourceId]++;
          g_coHdlcRxBytes[pEventData->m_nSourceId] += nFrameLen;
          if (nFrameEndCode) g_coHdlcBadRxFrames[pEventData->m_nSourceId]++;
        } else {
          DemoLogf("HDLC Frame Received with an invalid m_nSourceId (%d)\n", pEventData->m_nSourceId);
        }

        nDev = pEventData->m_nSourceId;

        if (g_bHdlcVerbose) {
          char strBuf[1024];
          //DemoLogf("HDLC Frame Received, Sequence Number = %d\n", pEventData->m_nParam);
          
          sprintf(strBuf, "HdlcReceiver[%d] Len=%d, Fe=0x%.2X, Sq=%d: ", pEventData->m_nSourceId, nFrameLen, nFrameEndCode, nFrameSeq);

          //DemoLogf("[%d] Len=%.3d, Fe=0x%.2X   ", pEventData->m_nSourceId, nFrameLen, nFrameEndCode);
          for (i = 0; i < nFrameLen && i < DEMO_HDLC_RX_BUFFER_SIZE; i++) {
            //sprintf(&strBuf[strlen(strBuf)], "%.2X ", buf[i]);
            // FISU
            if (nFrameLen == 3){
              if (pEventData->m_nSourceId == 0){

                // when the new FISU comes in, save current FISU to previous FISU array
                g_nPreviousFisu0[i] = g_nCurrentFisu0[i];
                // save new FISU
                g_nCurrentFisu0[i] = buf[i];
                DemoLogf("%.2X ", g_nCurrentFisu0[i]);

              } else {
                g_nPreviousFisu1[i] = g_nCurrentFisu1[i];
                g_nCurrentFisu1[i] = buf[i];
                DemoLogf("%.2X ", g_nCurrentFisu1[i]);
              }
            }
              
            // LSSU
            if (nFrameLen == 4){
              if (pEventData->m_nSourceId == 0){
                  
                // when the new LSSU comes in, save current LSSU to previous LSSU array
                g_nPreviousLssu0[i] = g_nCurrentLssu0[i];
                // save new LSSU
                g_nCurrentLssu0[i] = buf[i];
                DemoLogf("%.2X ", g_nCurrentLssu0[i]);

              } else {
                g_nPreviousLssu1[i] = g_nCurrentLssu1[i];
                g_nCurrentLssu1[i] = buf[i];
                DemoLogf("%.2X ", g_nCurrentLssu1[i]);
              }
            }
          }
          sprintf(&strBuf[strlen(strBuf)], "\n");

          DemoLogf(strBuf);
          
        }

        if (g_bSs7SimulationActive[nDev]) {
          if (nFrameEndCode == 0) {
            OTX_BOOL bErroredFrame = OTX_FALSE;
            
            pFisu = (OtxHdlcSS7FisuS *)&buf[0];
            
            if (((OTX_UINT8)pFisu->fsn != g_sqDspHdlcReceiver[nDev].nPrevFsn) && ((OTX_UINT8)pFisu->fsn != ((g_sqDspHdlcReceiver[nDev].nPrevFsn+1) % 128))) {
              bErroredFrame = OTX_TRUE;
              g_sqDspHdlcReceiver[nDev].nSs7Errors++;
              DemoPrintTimeStamp(pEventData->m_nTimeStamp);
              DemoLogf("=============v Fsn Error (%d) FSN=%d, prevFSN=%d\n", g_sqDspHdlcReceiver[nDev].nSs7Errors, pFisu->fsn, g_sqDspHdlcReceiver[nDev].nPrevFsn);
              DemoLogf("%d:Len=%.3d  ", pEventData->m_nSourceId, nFrameLen);
              for (i = 0; i < nFrameLen && i < DEMO_HDLC_RX_BUFFER_SIZE; i++) {
                DemoLogf("%.2X ", buf[i]);
              }
              DemoLogf("\n");

            }
            
            if (nFrameLen < 3) {
              DemoPrintTimeStamp(pEventData->m_nTimeStamp);
              DemoLogf("HDLC[%d] Received a too short frame\n", nDev);
            } else if (nFrameLen == 3) {
              // FISU, No action needed
            } else {
              // MSU, Acknowledge the frame by changing the FISU
              
              g_nSs7SimulationFramesReceived[nDev]++;


              OsaMutexLock(&g_demoSendSs7DataCritSec);

              if (bErroredFrame) {
                // Leave the BSN number the same, and toggle bib
                if (g_sqDspHdlcSender[nDev].fisu.bib) {
                  g_sqDspHdlcSender[nDev].fisu.bib = 0;
                } else {
                  g_sqDspHdlcSender[nDev].fisu.bib = 1;
                }

              } else {
                g_sqDspHdlcSender[nDev].fisu.bsn = pFisu->fsn;
                g_sqDspHdlcSender[nDev].fisu.bib = pFisu->fib;
              }
              g_sqDspHdlcSender[nDev].fisu.li = 0;
                            
              result = OtxHdlcSS7SendData(g_sqDspHdlcSender[nDev].hHandle, (OTX_UINT8 *)&g_sqDspHdlcSender[nDev].fisu, 3, &g_sqDspHdlcSender[nDev].fisu, DEMO_TASK_REF, OTX_TIME_INFINITY); 

              OsaMutexUnlock(&g_demoSendSs7DataCritSec);

              if (OTX_RESULT_CODE(result) == OTX_E_BUSY) {
                DemoLogf("HDLC[%d] Driver IO Request buffer is full. We need to retransmit this frame [not yet implemented]\n", nDev);
                // Driver IO Request buffer is full. We need to retransmit this frame.
              } else {
                if (OTX_FAILURE(result)) DemoLogf("HDLC[%d] OtxHdlcSS7SendData() failed. hHandle=%X\n", nDev, pEventData->m_hDevice);
                DemoCheckRc(g_sqDspHdlcSender[nDev].hHandle, result);
              }


            }
                    
            g_sqDspHdlcReceiver[nDev].nPrevFsn = pFisu->fsn;
                    
          } else {
            g_sqDspHdlcReceiver[nDev].nSs7Errors++;
            DemoPrintTimeStamp(pEventData->m_nTimeStamp);
            DemoLogf("%d:Len=%.3d  Bad Frame\n", pEventData->m_nSourceId, nFrameLen);
          }
        
        }
        otxRc = OtxHdlcReadData(pEventData->m_hDevice, buf, DEMO_HDLC_RX_BUFFER_SIZE, &nFrameLen, &nFrameSeq, &nFrameEndCode, &nTimeStamp);
      }
      if (OTX_RESULT_CODE(otxRc) != OTX_S_FALSE) {
        DemoLogf("HdlcReceiver[%d] OtxHdlcReadData() was unsuccessful.\n", pEventData->m_nSourceId);
        DemoCheckRc(pEventData->m_hDevice, otxRc);
      }
    } else if (pEventData->m_nCode == OTX_HDLC_RECEIVER_EC_SS7_FISU_COUNT) {
      
      OTX_UINT8 i = 0;

      DemoLogf("HdlcReceiver[%d] previous FISU:", pEventData->m_nSourceId);
                
      for (i = 0; i < 3; i++) { 
        if (pEventData->m_nSourceId == 0){
          DemoLogf("%.2X ", g_nPreviousFisu0[i]);
        } else {      
          DemoLogf("%.2X ", g_nPreviousFisu1[i]);
        }
      }

      DemoLogf("\nHdlcReceiver[%d] previous FISU count = %d\n", pEventData->m_nSourceId, pEventData->m_nParam - 1); // pEventData->m_nParam is the count of previous FISU, for the first FISU, there is no previous FISU, so minus 1 
  
    } else if (pEventData->m_nCode == OTX_HDLC_RECEIVER_EC_SS7_LSSU_COUNT) {
      
      OTX_UINT8 i = 0;

      DemoLogf("HdlcReceiver[%d] previous LSSU:", pEventData->m_nSourceId);
                
      for (i = 0; i < 4; i++) { 
        if (pEventData->m_nSourceId == 0){
          DemoLogf("%.2X ", g_nPreviousLssu0[i]);
        } else {      
          DemoLogf("%.2X ", g_nPreviousLssu1[i]);
        }
      }

      DemoLogf("\nHdlcReceiver[%d] previous LSSU count = %d\n", pEventData->m_nSourceId, pEventData->m_nParam - 1); // pEventData->m_nParam is the count of previous LSSU, for the first FISU, there is no previous LSSU, so minus 1 

    } else {
      // Unexpected event from this device
      otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
      if (OTX_SUCCESS(otxRc)) {
        DemoLogf("HdlcReceiver[%d] Event: %s \n", pEventData->m_nSourceId, msgStr);
      } else {
        DemoLogf("HdlcReceiver[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
      }
    }
    break;

  case OTX_LDEVICE_HDLC_SENDER:
    otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
    DemoLogf("HdlcSender[%d] Event: %s \n", pEventData->m_nSourceId, msgStr);
    break;

  default:
    // Unhandled event
    DemoLogf("Event from an unknown object 0x%X\n", pEventData->m_hDevice);
    break;
  }


}




/*******************************************************************************

  @func

  Transmits a number of MSU frames

  @comm

*******************************************************************************/
void *DemoSs7SimulationFunction(
  void *lpParameter   // thread data
)
{
  OTX_DATETIME nTimeStamp;
  OTX_RESULT result;
  OTX_INT32 i;
  clock_t startTime, stopTime;
  OtxHdlcSS7FisuS msu;
  OTX_UINT8 sqHdlcFrame[300];
  OTX_INT32 cbHdlcFrame;
  OTX_UINT32 nDev;
  OTX_INT32 coHdlcSenderFrames = g_demoEnv.nSimFrames;

  nDev = *((OTX_INT32 *) lpParameter);

  g_sqDspHdlcSender[nDev].busyCount = 0;

  g_sqDspHdlcSender[nDev].coMsusSent = 0;
  g_sqDspHdlcSender[nDev].coFisusSent = 0;

  g_sqDspHdlcSender[nDev].fisu.bib = 1;
  g_sqDspHdlcSender[nDev].fisu.fib = 1;
  g_sqDspHdlcSender[nDev].fisu.fsn = 0x7F;
  g_sqDspHdlcSender[nDev].fisu.bsn = 0x7F;
  g_sqDspHdlcSender[nDev].fisu.li = 0;
  g_sqDspHdlcSender[nDev].fisu.reserved = 0;
  

  g_bSs7SimulationActive[nDev] = OTX_TRUE;

  g_nSs7SimulationFramesReceived[nDev] = 0;

  srand(g_sqDspHdlcSender[nDev].hHandle);

  memcpy(sqHdlcFrame, &g_sqDspHdlcSender[nDev].fisu, OTX_HDLC_SS7_CB_FISU);  
  cbHdlcFrame = OTX_HDLC_SS7_CB_FISU;
  
  DemoLogf("Sending %d MSU frames from HDLC Sender #%d  (Dsp#%d, Li#%d)\n", coHdlcSenderFrames, nDev, g_sqDspHdlcSender[nDev].nDsp, g_sqDspHdlcSender[nDev].nLi);

  result = OtxHdlcStartRepeatData(g_sqDspHdlcSender[nDev].hHandle, (OTX_UINT8 *) &g_sqDspHdlcSender[nDev].fisu, OTX_HDLC_SS7_CB_FISU, DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(g_sqDspHdlcSender[nDev].hHandle, result);

  OsaThreadSleep(50);
  
  // Send MSU Frame(s)
  startTime = clock();
  while (g_sqDspHdlcSender[nDev].coMsusSent < coHdlcSenderFrames) {

    if (g_sqDspHdlcSender[nDev].coMsusSent == 0) {
      DemoLogf("%d: hHandle=%X\n", nDev, g_sqDspHdlcSender[nDev].hHandle);
    }

    OsaMutexLock(&g_demoSendSs7DataCritSec);

    // Step FSN for every new frame package (MSU+FISU or FISU+FISU) to send
    g_sqDspHdlcSender[nDev].fisu.fsn++;
    
    // Prepare an MSU
    //cbHdlcFrame = ((g_sqDspHdlcSender[nDev].coMsusSent%10)*10) + 10;
    if ((rand() % 10) < 3) {
      // Longer frames
      cbHdlcFrame = rand() % (272-4+1) + 4;
    } else {
      // Shorter frames
      cbHdlcFrame = rand() % (25-4) + 4;
    }
    memcpy(&msu, &g_sqDspHdlcSender[nDev].fisu, OTX_HDLC_SS7_CB_FISU); // Put the first 3 bytes of the MSU in the HDLC frame buffer
    msu.li = 3;  // Set the Length Indicator; Anything larger than 2 is an MSU
    memcpy(sqHdlcFrame, &msu, OTX_HDLC_SS7_CB_FISU); // Put the first 3 bytes of the MSU in the HDLC frame buffer
    for (i = OTX_HDLC_SS7_CB_FISU; i < cbHdlcFrame; i++) {     // Add the payload the HDLC frame buffer
      //sqHdlcFrame[i] = (OTX_UINT8) i;
      sqHdlcFrame[i] = (OTX_UINT8) rand();
    }

    result = OtxHdlcSS7SendData(g_sqDspHdlcSender[nDev].hHandle, sqHdlcFrame, cbHdlcFrame, &g_sqDspHdlcSender[nDev].fisu, DEMO_TASK_REF, OTX_TIME_INFINITY); 

    OsaMutexUnlock(&g_demoSendSs7DataCritSec);

    if (OTX_RESULT_CODE(result) == OTX_E_BUSY) {
      OtxDrvGetCurrentTime(&nTimeStamp);
      DemoPrintTimeStamp(nTimeStamp);
      DemoLogf("%d: Driver IO Request buffer is full. We need to retransmit this frame. [not implemented]\n", nDev);
      g_sqDspHdlcSender[nDev].busyCount++;
      OsaThreadSleep(1);
    } else {
      DemoCheckRc(g_sqDspHdlcSender[nDev].hHandle, result);

      OsaThreadSleep(cbHdlcFrame/8 + rand() % 40);

      DemoLogf("%d: Sent MSU# %d\n", nDev, g_sqDspHdlcSender[nDev].coMsusSent);
      g_sqDspHdlcSender[nDev].coMsusSent++;
      
    }
  }
  stopTime = clock();
  
  //DemoLogf("[%d]Done (BusyCount=%d, time=%2.1f seconds) FISUs=%d MSUs=%d\n", nDev, g_sqDspHdlcSender[nDev].busyCount, (double)(stopTime - startTime) / CLOCKS_PER_SEC, g_sqDspHdlcSender[nDev].coFisusSent, g_sqDspHdlcSender[nDev].coMsusSent);
  DemoLogf("We are done. Exiting thread.\n");

#if 1
  while (1) {
    OsaThreadSleep(500);
  }
#endif

  return 0;
}






/*******************************************************************************

  @func

  Parse the program's command line arguments and calls upon
  requested actions.

  There are some optional switches that can be used following
  the driver name on the command line. Switches are are
  separated by spaces, and can appear in arbitrary order.
  Switches consist of a hyphen and a letter:

  @comm

*******************************************************************************/
static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP) {
  short  i;

  // Set reasonable defaults before reading from command line
  envP->pszInputStr       = NULL;
  envP->bExitOnError      = OTX_FALSE;
  envP->logFileName       = DEMO_DEFAULT_LOG_FILE;
  envP->eT1E1Mode         = OTX_T1E1_MODE_E1;
  envP->bSs7FisuFiltering = OTX_TRUE;
  envP->boardNo           = DEMO_DEFAULT_BOARD_NO;
  envP->nBoardType        = OTX_DEVICE_THOR_PCI;
  envP->coHdlcPerLi       = 1; 
  envP->coLi              = 2;
  envP->bUseDspDebugger   = OTX_FALSE;
  envP->coDsp             = 4;
  envP->bHdlcBitInversion = OTX_FALSE;
  envP->b56kbpsChannel    = OTX_FALSE;
  envP->nTimeSlots        = 11;
  envP->eProgramModule    = OTX_DSP_SPM_OTXSPM1;
  envP->eT1LineCode       = OTX_T1_LC_B8ZS;
  envP->eT1FrameFormat    = OTX_T1_FF_ESF;
  envP->eE1LineCode       = OTX_E1_LC_HDB3;
  envP->eE1FrameFormat    = OTX_E1_FF_CRC4_MULTIFRAME;
  envP->bUseSenders       = OTX_TRUE;
  envP->bUseReceivers     = OTX_TRUE;
  envP->eClkSource        = OTX_CLOCK_SOURCE_INTERNAL;
  envP->nSimFrames        = DEMO_DEFAULT_SS7_SIM_FRAMES;

  for (i = 1; i < argc; i++) {        // Scan command line arguments
    if (*(argv[i]+1) == 'Z') {   // Command input string -- case sensitive
      envP->pszInputStr = argv[i]+2;
    } else {    
      OsaStdStrUpr(argv[i]);
      if (*argv[i] == '-' || *argv[i] == '/') {  // Is argument a legal switch?
        if (*(argv[i]+1) == 'L') {   // Log file
          envP->logFileName = argv[i]+2;
        } else if (*(argv[i]+1) == 'Q') {   // Exit on error
          envP->bExitOnError = OTX_TRUE;
        } else if (*(argv[i]+1) == 'B') {   // Board number
          envP->boardNo = strtoul(argv[i]+2, NULL, 0);
        } else if (*(argv[i]+1) == 'S') {   // Sim frames
          envP->nSimFrames = strtoul(argv[i]+2, NULL, 0);
        } else if (*(argv[i]+1) == 'D') {   // DSP Debugger
          envP->bUseDspDebugger = OTX_TRUE;
        } else if (*(argv[i]+1) == 'I') {   // HDLC Bit Inversion
          envP->bHdlcBitInversion = OTX_TRUE;
        } else if (*(argv[i]+1) == '5') {   // Use 56kbps HDLC channel (vs 64kpbs)
          envP->b56kbpsChannel = OTX_TRUE;
        } else if (*(argv[i]+1) == 'N') {   // Number of timeslots in the superchannel
          envP->nTimeSlots = strtoul(argv[i]+2, NULL, 0);
        } else if (*(argv[i]+1) == 'M') {   // Li Mode
          if (*(argv[i]+2) == 'E') {   // E1 
            envP->eT1E1Mode    = OTX_T1E1_MODE_E1;
          } else if (*(argv[i]+2) == 'T') {   // T1
            envP->eT1E1Mode    = OTX_T1E1_MODE_T1;
          } else {
            DemoLogf("Error: Invalid Li Mode (E1/T1) command line switch %s\n", argv[i]);
            DemoHelpPanel();
            OsaConsoleExit();
					  exit(1);
          }
        } else if (*(argv[i]+1) == 'K') {   // Clock Source
          if (*(argv[i]+2) == 'I') {   // Internal
            envP->eClkSource = OTX_CLOCK_SOURCE_INTERNAL;
          } else if (*(argv[i]+2) == '0') {   // Line Interface 0
            envP->eClkSource = OTX_CLOCK_SOURCE_LOCAL_0;
          } else if (*(argv[i]+2) == '1') {   // Line Interface 1
            envP->eClkSource = OTX_CLOCK_SOURCE_LOCAL_1;
          } else if (*(argv[i]+2) == 'E') {   // External
            envP->eClkSource = OTX_CLOCK_SOURCE_LOCAL_2;
          } else if (*(argv[i]+2) == '3') {   // Split clock, both Lis synchronized to its own line
            envP->eClkSource = OTX_CLOCK_SOURCE_LOCAL_3;
          } else if (*(argv[i]+2) == '4') {   // Split clock, Li0 synchronized to its T1E1 span. Li1 internal
            envP->eClkSource = OTX_CLOCK_SOURCE_LOCAL_4;
          } else if (*(argv[i]+2) == '5') {   // Split clock, Li1 synchronized to its T1E1 span. Li0 internal
            envP->eClkSource = OTX_CLOCK_SOURCE_LOCAL_5;
          } else {
            DemoLogf("Error: Invalid Clock Source command line switch %s\n", argv[i]);
            DemoHelpPanel();
            exit(1);
          }
        } else if (*(argv[i]+1) == 'P') {   // Dsp Program module
          if (*(argv[i]+2) == '1') {          // OtxSpm1.out
            envP->eProgramModule = OTX_DSP_SPM_OTXSPM1;
          } else if (*(argv[i]+2) == '2') {   // OtxSpm2.out
            envP->eProgramModule = OTX_DSP_SPM_OTXSPM2;
          } else if (*(argv[i]+2) == 'H') {   // OtxHdlc.out
            envP->eProgramModule = OTX_DSP_SPM_OTXHDLC;
          } else if (*(argv[i]+2) == 'E') {   // OtxHdlcEx.out
            envP->eProgramModule = OTX_DSP_SPM_OTXHDLCEX;
          } else {
            DemoLogf("Error: Invalid DSP Program Module command line switch %s\n", argv[i]);
            DemoHelpPanel();
            exit(1);
          }
        } else if (*(argv[i]+1) == 'T') {   // Board Type
          if ( *(argv[i]+2) == '2') {
            envP->nBoardType = OTX_DEVICE_THOR_PCI;
            envP->coLi = 2;
            envP->coDsp = 4;
            envP->coHdlcPerLi = 12; 
          } else if ( *(argv[i]+2) == 'M') {
            envP->nBoardType  = OTX_DEVICE_THOR_PCMCIA;
            envP->coLi = 2;
            envP->coDsp = 2;
            //envP->coHdlcPerLi = 6; 
            envP->coHdlcPerLi = 1; 
          } else if ( *(argv[i]+2) == '4') {
            envP->nBoardType  = OTX_DEVICE_THOR_4_PCI_PLUS;
            envP->coLi = 2;
            envP->coDsp = 2;
            envP->coHdlcPerLi = 1; 
          } else if ( *(argv[i]+2) == 'T') {
            envP->nBoardType  = OTX_DEVICE_THOR_2_PCI_PLUS;
            envP->coLi = 2;
            envP->coDsp = 2;
            envP->coHdlcPerLi = 1; 
          } else if ( *(argv[i]+2) == 'E') {
            envP->nBoardType  = OTX_DEVICE_THOR_2_PCI_EXPRESS;
            envP->coLi = 2;
            envP->coDsp = 2;
            envP->coHdlcPerLi = 1; 
          } else if ( *(argv[i]+2) == '8') {
            envP->nBoardType  = OTX_DEVICE_THOR_8_PCI;
            envP->coLi = 8;
            envP->coDsp = 4;
            envP->coHdlcPerLi = 1; 
          } else if ( *(argv[i]+2) == 'P') {
            envP->nBoardType  = OTX_DEVICE_THOR_8_PCI_PLUS;
            envP->coLi = 8;
            envP->coDsp = 2;
            envP->coHdlcPerLi = 1; 
          } else if ( *(argv[i]+2) == 'G') {
            envP->nBoardType  = OTX_DEVICE_GIMLE_16_PCI_PLUS;
            envP->coLi = 2;
            envP->coDsp = 2;
            envP->coHdlcPerLi = 1; 
            envP->bUseSenders = OTX_FALSE;
          } else if ( *(argv[i]+2) == 'S') {
            envP->nBoardType  = OTX_DEVICE_SLEIPNIR_1_PCI;
            envP->coLi = 1;
            envP->coDsp = 1;
            envP->coHdlcPerLi = 1; 
          } else {
            DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
            DemoHelpPanel();
            exit(1);
          }
        
        } else if (*(argv[i]+1) == 'F') {   // T1 Frame format
          if ( *(argv[i]+2) == 'E') {
            envP->eT1FrameFormat = OTX_T1_FF_ESF; // ESF
          } else if ( *(argv[i]+2) == '4') {
            envP->eT1FrameFormat = OTX_T1_FF_F4; 
          } else if ( *(argv[i]+2) == 'S') {
            envP->eT1FrameFormat = OTX_T1_FF_F12; // SF
          } else if ( *(argv[i]+2) == '9') {
            envP->eT1FrameFormat = OTX_T1_FF_F72; // SLC96
          } else if ( *(argv[i]+2) == 'J') {
            envP->eT1FrameFormat = OTX_T1_FF_ESF_J1; // J1_ESF
          } else if ( *(argv[i]+2) == '5') {
            envP->eT1FrameFormat = OTX_T1_FF_F4_J1; 
          } else if ( *(argv[i]+2) == 'F') {
            envP->eT1FrameFormat = OTX_T1_FF_F12_J1; // J1_SF
          } else if ( *(argv[i]+2) == '0') {
            envP->eT1FrameFormat = OTX_T1_FF_F72_J1; // J1_SLC96
          } else {
            DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
            DemoHelpPanel();
            OsaConsoleExit();
            exit(1);
          }
          
        } else if (*(argv[i]+1) == 'J') {   // T1 Line Code
          if ( *(argv[i]+2) == 'A') {
            envP->eT1LineCode = OTX_T1_LC_AMI; // AMI
          } else if ( *(argv[i]+2) == 'B') {
            envP->eT1LineCode = OTX_T1_LC_B8ZS; // B8ZS
          } else {
            DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
            DemoHelpPanel();
            OsaConsoleExit();
            exit(1);
          }
          
        } else if (*(argv[i]+1) == 'G') {   // E1 Frame format
          if ( *(argv[i]+2) == 'D') {
            envP->eE1FrameFormat = OTX_E1_FF_DOUBLE_FRAME; 
          } else if ( *(argv[i]+2) == 'C') {
            envP->eE1FrameFormat = OTX_E1_FF_CRC4_MULTIFRAME; 
          } else if ( *(argv[i]+2) == 'G') {
            envP->eE1FrameFormat = OTX_E1_FF_CRC4_MULTIFRAME_G706; 
          } else {
            DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
            DemoHelpPanel();
            OsaConsoleExit();
            exit(1);
          }
          
        } else if (*(argv[i]+1) == 'U') {   // E1 Line Code
          if ( *(argv[i]+2) == 'A') {
            envP->eE1LineCode = OTX_E1_LC_AMI; // AMI
          } else if ( *(argv[i]+2) == 'H') {
            envP->eE1LineCode = OTX_E1_LC_HDB3; // HDB3
          } else {
            DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
            DemoHelpPanel();
            OsaConsoleExit();
            exit(1);
          }
          
        } else {
          DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
          DemoHelpPanel();
          OsaConsoleExit();
				  exit(1);
        }
      } else {
        DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
        DemoHelpPanel();
        OsaConsoleExit();
			  exit(1);
      }
    }
  }

  if (g_demoEnv.b56kbpsChannel) {
    envP->coHdlcPerLi       = 1;
  }

}


/*******************************************************************************

  @func

  Displays a help for the command line.

  @comm

*******************************************************************************/
static void DemoHelpPanel()
{
  DemoLogf("\nCommand line switches:\n");
  DemoLogf(" /B<boardNo>   : Board number (Default is %d)\n", DEMO_DEFAULT_BOARD_NO);
  DemoLogf(" /T[2|T|E|M|4|8|P|G|S]  : Board Type used for the demo, where\n");
  DemoLogf("                      2 = Use Thor-2-PCI Board with Vidar-ASM\n");
  DemoLogf("                      T = Use Thor-2-PCI-Plus with Vidar-ASM Board\n");
  DemoLogf("                      E = Use Thor-2-PCI-Express with Vidar-ASM Board\n");
  DemoLogf("                      M = Use Thor-2-PCMCIA Board\n");
  DemoLogf("                      4 = Use Thor-4-PCI-Plus with Vidar-ASM Board\n");
  DemoLogf("                      8 = Use Thor-8-PCI with Vidar-ASM Board\n");
  DemoLogf("                      P = Use Thor-8-PCI-Plus with Vidar-ASM Board\n");
  DemoLogf("                      G = Use Gimle-16-PCI-Plus with Vidar-ASM Board\n");
  DemoLogf("                   S = Use Sleipnir-1-PCI (DSP version)\n");
  DemoLogf(" /L<fileName>  : Log file name\n");
  DemoLogf(" /S            : SS7 FISU filtering active\n");
  DemoLogf(" /ME           : E1 Li mode\n");
  DemoLogf(" /MT           : T1/J1 Li mode\n");
  DemoLogf(" /D            : DSP Debugger mode. Prompt to start DSP code from Debugger\n");
  DemoLogf(" /I            : Use HDLC Bit Inversion (Default is to not invert the bits)\n");
  DemoLogf(" /5            : Use HDLC 56kbps channel.\n");
  DemoLogf(" /N<timeslots> : Number of 64kbps timeslots in the Super-channels (only used if /X is used)\n");
  DemoLogf(" /P[1|2|C|H|E] : DSP program to load, where\n");
  DemoLogf("                   1 = OtxSpm1\n");
  DemoLogf("                   2 = OtxSpm2\n");
  DemoLogf("                   H = OtxHdlc (default)\n");
  DemoLogf("                   E = OtxHdlcEx (for 5416 and 5510 DSPs)\n");
  DemoLogf(" /F[E|4|S|9|J|5|F|0] : T1/J1 Frame Format, where\n");
  DemoLogf("                   E = ESF (default)\n");
  DemoLogf("                   4 = F4\n");
  DemoLogf("                   S = SF [D3/D4] (F12)\n");
  DemoLogf("                   9 = SLC96 (F72)\n");
  DemoLogf("                   J = J1_ESF\n");
  DemoLogf("                   5 = J1_F4\n");
  DemoLogf("                   F = J1_SF\n");
  DemoLogf("                   0 = J1_SLC96\n");
  DemoLogf(" /J[A|B]       : T1/J1 Line Code, where\n");
  DemoLogf("                   A = AMI\n");
  DemoLogf("                   B = B8ZS (default)\n");
  DemoLogf(" /G[D|C|G]     : E1 Frame Format, where\n");
  DemoLogf("                   D = Doubleframe\n");
  DemoLogf("                   C = CRC4 Multiframe (default)\n");
  DemoLogf("                   G = CRC4 Multiframe (G.706)\n");
  DemoLogf(" /U[A|H]       : E1 Line Code, where\n");
  DemoLogf("                   A = AMI\n");
  DemoLogf("                   H = HDB3 (default)\n");
  DemoLogf(" /Z<str>       : Command string (overrides keyboard input)\n");
  DemoLogf(" /Q            : Exit without prompting on errors\n");
  DemoLogf(" /K[I|0|1]     : Clock source, where\n");
  DemoLogf("                   I = Internal on-board oscillator\n");
  DemoLogf("                   0 = Recovered clock from Line Interface 0\n");
  DemoLogf("                   1 = Recovered clock from Line Interface 1\n");
}


/*******************************************************************************

  @func

  Displays a message on the screen and prints it to the logfile.

  @comm

*******************************************************************************/
static void DemoLogf(char *formatStr, ...)
{
  va_list argList;

  OsaMutexLock(&g_demoLogfCritSec);

  va_start(argList, formatStr);

  OsaConsolePrintF(formatStr, argList);
  if (g_pDemoLogFile) vfprintf(g_pDemoLogFile,   formatStr, argList);

  va_end(argList);

  OsaMutexUnlock(&g_demoLogfCritSec);

}

static void DemoCheckRc(IN OTX_HANDLE hDevice, IN OTX_RESULT result)
/*******************************************************************************
  @func
    Check the result from API call. If the result is a failure code then prompt
    the user if he wants to exit.
*******************************************************************************/
{
  if ( OTX_FAILURE(result) ) {

    if (hDevice == OTX_INVALID_HANDLE_VALUE) {
      DemoLogf("ERROR(%#lX) \n", result);
    } else {
      OTX_RESULT otxRc;
      char msgStr[256];
      OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
      OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
      OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
      OTX_UINT32      eDeviceType = 999;    // Device type for hDevice 
      OTX_UINT32      nDeviceNumber = 999;  // Device number for hDevice
      
      // Get the Device Type for hHandle
      otxRc = OtxDrvGetAttributeType(hDevice, OTX_ATTR_DEVICE_TYPE, &eAttribType, &cbAttribValue, &bReadOnly); 
      if (OTX_SUCCESS(otxRc)) {
        assert(eAttribType == OTX_ATTRIB_UINT32);
        assert(cbAttribValue == sizeof(OTX_UINT32));
        OtxDrvGetAttributeValueUint32(hDevice, OTX_ATTR_DEVICE_TYPE, &eDeviceType); 
      } 
      
      // Get the Device Number for hHandle
      otxRc = OtxDrvGetAttributeType(hDevice, OTX_ATTR_DEVICE_NO, &eAttribType, &cbAttribValue, &bReadOnly); 
      if (OTX_SUCCESS(otxRc)) {
        assert(eAttribType == OTX_ATTRIB_UINT32);
        assert(cbAttribValue == sizeof(OTX_UINT32));
        OtxDrvGetAttributeValueUint32(hDevice, OTX_ATTR_DEVICE_NO, &nDeviceNumber); 
      }
      
      // Convert the result code to a more descriptive string
      otxRc = OtxDrvResultCode2String(OTX_DRIVER_DEVICE_HANDLE, result, 256, &msgStr[0]);
      if (OTX_FAILURE(otxRc)) {
        strcpy(msgStr, "Cannot translate this result code.");
      }
      DemoLogf("ERROR(%#lX [%d.%d]) Dev(%X,%d,%d): %s \n", OTX_RESULT_CODE(result), OTX_RESULT_SOURCE_ID(result), OTX_RESULT_LINE_NUM(result), hDevice, (int) eDeviceType, (int)nDeviceNumber, msgStr);
    }

    DemoExitQuestion();
  }
}


/*******************************************************************************
  @func
    Prompts whether to exit the program. Usually called after an error 
    has been reported.

*******************************************************************************/
void DemoExitQuestion()
{
  short c;

  if (g_demoEnv.bExitOnError) {
    DemoLogf("\nExitOnError....\n");
    c = 'y';
  } else {
    DemoLogf("Do you want to exit (Y/N) ");
    c = OsaConsoleGetCh();
  }
  if (c == 'y' || c == 'Y') {
    // Clean up
    OtxDrvReleaseLib();
    OsaConsoleExit();
    exit(1);
  }
}


/*******************************************************************************

  @func

  Processes User Commands

  @comm

*******************************************************************************/
OTX_INT32 DemoProcessUserCommand(char commandChar)
{
  static OTX_INT32      nCurrentHdlcDev = 0;
  OTX_RESULT result;

  DemoLogf("\nKey %c pressed\n", commandChar);

  switch (commandChar) {

  case '?':
  case 'h':
  case 'H':
    DemoLogf("Cmd  Synopsis\n");
    DemoLogf("===  ========\n");
    DemoLogf(" ?   Display Help text\n");
    DemoLogf(" h   Ditto\n");
    DemoLogf(" q   Quit\n");
    if (g_demoEnv.bUseSenders) {
      DemoLogf(" t   Start SS7 simulation on Li0 and Li1\n");
      DemoLogf(" F   Toggle OTX_HDLC_SS7_FILTER_FISU_ALL filter mask on/off for current HDLC Sender\n");
      DemoLogf(" f   Start sending FISUs (different FISUs from last 'f') continuosly from current HDLC Sender\n");
      DemoLogf(" z   Start sending FISUs (same FISUs as last 'z') continuosly from current HDLC Sender\n");
      DemoLogf(" i   Start sending LSSUs continuosly from current HDLC Sender\n");
    }
    DemoLogf(" c   Set Clock Source\n");
    DemoLogf(" l   Query Line Status of the two Lis\n");
    DemoLogf(" s   Display the SS7 HDLC receive status of the active HDLC Receiver\n");
    DemoLogf(" n   Increment current HDLC Device (currently HdlcDev#%d selected\n", nCurrentHdlcDev);
    DemoLogf(" N   Decrement current HDLC Device (currently HdlcDev#%d selected\n", nCurrentHdlcDev);
    DemoLogf(" y   Query HDLC Receiver Fifo diagnostics (for current HDLC device)\n");
    DemoLogf(" p   Reset current HDLC Receiver\n");
    DemoLogf(" v   Toggle verbose logging of received HDLC frames\n");
    DemoLogf(" V   Print counters of received HDLC frames\n");
    break;

  case 'x':
    {
      char szBuf[12];
      OTX_INT32 nDsp;                                                             
      
      DemoLogf("Dump profiling data. Which Dsp#?");
      OsaConsoleGetStr(szBuf);
      nDsp = (OTX_INT32) strtol(szBuf, NULL, 0);
      
      OtxDrvDbgDump(g_hPhysDevs.hDsp[nDsp], 3);
    }
    break;
  case 'j':
    {
      char szBuf[12];
      OTX_INT32 nDsp;
      
      DemoLogf("Dump debug data. Which Dsp#?");
      OsaConsoleGetStr(szBuf);
      nDsp = (OTX_INT32) strtol(szBuf, NULL, 0);
      
      OtxDrvDbgDump(g_hPhysDevs.hDsp[nDsp], 0);
    }
    break;
  case 'J':
    {
      OTX_INT32 nDsp;
      
      for (nDsp = 0; nDsp < g_demoEnv.coDsp; nDsp++) {
        DemoLogf("Dump debug data for Dsp#%d\n", nDsp);
        OtxDrvDbgDump(g_hPhysDevs.hDsp[nDsp], 0);
      }
    }
    break;
  case 'B':
    {
      OTX_INT32 nDsp;
      
      for (nDsp = 0; nDsp < g_demoEnv.coDsp; nDsp++) {
        DemoLogf("Init Bsp for Dsp#%d\n", nDsp);
        OtxDrvDbgDump(g_hPhysDevs.hDsp[nDsp], 1);
      }
    }
    break;
  case 'k':
    {
      OTX_UINT32 nAttribValue;
      OTX_INT32 nDsp = 0;
      DemoLogf("Notication Event Buffer Diagnostics for DSP#%d:\n", nDsp);

      result = OtxDrvGetAttributeValueUint32(g_hPhysDevs.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_SIZE, &nAttribValue); DemoCheckRc(g_hPhysDevs.hDsp[nDsp], result);
      DemoLogf("  Size (in words)         : %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_hPhysDevs.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_CURRENT_USAGE, &nAttribValue); DemoCheckRc(g_hPhysDevs.hDsp[nDsp], result);
      DemoLogf("  Current usage (in words): %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_hPhysDevs.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_MAX_USAGE, &nAttribValue); DemoCheckRc(g_hPhysDevs.hDsp[nDsp], result);
      DemoLogf("  Max usage (in words)    : %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_hPhysDevs.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_LOST_EVENTS, &nAttribValue); DemoCheckRc(g_hPhysDevs.hDsp[nDsp], result);
      DemoLogf("  Number of lost events   : %d\n", nAttribValue);
    }
    break;
  case 'K':
    {
      OTX_UINT32 nAttribValue;
      OTX_INT32 nDsp = 1;
      DemoLogf("Notication Event Buffer Diagnostics for DSP#%d:\n", nDsp);

      result = OtxDrvGetAttributeValueUint32(g_hPhysDevs.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_SIZE, &nAttribValue); DemoCheckRc(g_hPhysDevs.hDsp[nDsp], result);
      DemoLogf("  Size (in words)         : %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_hPhysDevs.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_CURRENT_USAGE, &nAttribValue); DemoCheckRc(g_hPhysDevs.hDsp[nDsp], result);
      DemoLogf("  Current usage (in words): %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_hPhysDevs.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_MAX_USAGE, &nAttribValue); DemoCheckRc(g_hPhysDevs.hDsp[nDsp], result);
      DemoLogf("  Max usage (in words)    : %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_hPhysDevs.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_LOST_EVENTS, &nAttribValue); DemoCheckRc(g_hPhysDevs.hDsp[nDsp], result);
      DemoLogf("  Number of lost events   : %d\n", nAttribValue);
    }
    break;

  case 'y':
    {
      OTX_UINT32 nAttribValue;
      DemoLogf("Fifo Diagnostics for HDLC Receiver device#%d:\n", nCurrentHdlcDev);

      result = OtxDrvGetAttributeValueUint32(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, OTX_ATTR_HDLC_RECEIVER_FIFO_SIZE_FRAMES, &nAttribValue); DemoCheckRc(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, result);
      DemoLogf("  Size (in frames)         : %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, OTX_ATTR_HDLC_RECEIVER_FIFO_SIZE_OCTETS, &nAttribValue); DemoCheckRc(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, result);
      DemoLogf("  Size (in octets)         : %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, OTX_ATTR_HDLC_RECEIVER_FIFO_PEAK_USAGE_FRAMES, &nAttribValue); DemoCheckRc(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, result);
      DemoLogf("  Peak usage (in frames)   : %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, OTX_ATTR_HDLC_RECEIVER_FIFO_PEAK_USAGE_OCTETS, &nAttribValue); DemoCheckRc(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, result);
      DemoLogf("  Peak usage (in octets)   : %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, OTX_ATTR_HDLC_RECEIVER_FIFO_CURRENT_USAGE_FRAMES, &nAttribValue); DemoCheckRc(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, result);
      DemoLogf("  Current usage (in frames): %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, OTX_ATTR_HDLC_RECEIVER_FIFO_CURRENT_USAGE_OCTETS, &nAttribValue); DemoCheckRc(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, result);
      DemoLogf("  Current usage (in octets): %d\n", nAttribValue);

      result = OtxDrvGetAttributeValueUint32(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, OTX_ATTR_HDLC_RECEIVER_FIFO_LOST_FRAMES, &nAttribValue); DemoCheckRc(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, result);
      DemoLogf("  Number of lost frames    : %d\n", nAttribValue);
    }
    break;


  case 'v':
    {
      g_bHdlcVerbose = !g_bHdlcVerbose;
      DemoLogf("g_bHdlcVerbose is now %s.\n", g_bHdlcVerbose ? "TRUE": "FALSE");
    }
    break;

  case 'S':
    {
      OTX_INT32 i;
      for (i = 0; i < 2; i++) {
          DemoLogf("RX Tot[%.2d]=%.6d (Bad[%.2d]=%.6d)   TX MSUs: %d FISUs: %d\n", 
            i, g_coHdlcRxFrames[i], i, g_coHdlcBadRxFrames[i],
            g_sqDspHdlcSender[i].coMsusSent, g_sqDspHdlcSender[i].coFisusSent
            );
      }
      DemoLogf("\n");
    }
    break;


  case 's':
    {
      OTX_INT32 i;
      OTX_UINT32  nFisuCount[8];   // Number of FISUs received
      OTX_UINT32  nLssuCount[8];   // Number of LSSUs received
      OTX_UINT32  nMsuCount[8];    // Number of MSUs received
      OtxHdlcSS7FisuS fisu[8];     // The last received FISU on the pipe.
      DemoLogf("SS7 Status for HdlcReceiver #%d - #%d\n", 0, 7);

      for (i = 0; i < g_demoEnv.coLi; i++) {
        result = OtxHdlcSS7GetReceiveStatus(g_sqDspHdlcReceiver[i].hHandle, &nFisuCount[i], &nLssuCount[i], &nMsuCount[i], &fisu[i]); DemoCheckRc(g_sqDspHdlcReceiver[i].hHandle, result);
      }

      DemoLogf("FISUs received: ");
      for (i = 0; i < g_demoEnv.coLi; i++) {
        DemoLogf("%.6lu ", nFisuCount[i]);
      }
      DemoLogf("\n");

      DemoLogf("LSSUs received: ");
      for (i = 0; i < g_demoEnv.coLi; i++) {
        DemoLogf("%.6lu ", nLssuCount[i]);
      }
      DemoLogf("\n");

      DemoLogf("MSUs received:  ");
      for (i = 0; i < g_demoEnv.coLi; i++) {
        DemoLogf("%.6lu ", nMsuCount[i]);
      }
      DemoLogf("\n");

      for (i = 0; i < g_demoEnv.coLi; i++) {
        DemoLogf("Last FISU%d received: bsn = %d, bib = %d, fsn = %d, fib = %d, li = %d\n", i,
               fisu[i].bsn, fisu[i].bib, fisu[i].fsn, fisu[i].fib, fisu[i].li);
      }

      for (i = 0; i < g_demoEnv.coLi; i++) {
          DemoLogf("RX Tot[%.2d]=%.6d (Bad[%.2d]=%.6d)   TX MSUs: %d busy: %d ERR=%d\n", 
            i, g_coHdlcRxFrames[i], 
            i, g_coHdlcBadRxFrames[i],
            g_sqDspHdlcSender[i].coMsusSent, 
            g_sqDspHdlcSender[i].busyCount,
            g_sqDspHdlcReceiver[i].nSs7Errors
            );
      }
      DemoLogf("\n");
    }
    break;


  // this command is only available on PRO cards with OTX_DSP_SPM_OTXSPM2
  // it checks DspHdlcReceiver*0* only.
  case 'G':
    {
      OTX_UINT8 i = 0;
      OTX_UINT32  nFisuCount0;      // Number of FISUs received
      OTX_UINT32  nLssuCount0;      // Number of LSSUs received
      OtxHdlcSS7FisuS       fisu0;  // The last received FISU on the pipe.
      OtxHdlcSS7ShortLssuS  lssu0;  // The last received LSSU on the pipe.
      DemoLogf("SS7 Status for HdlcReceiver #0\n");

      result = OtxHdlcSS7GetSignalCount(g_sqDspHdlcReceiver[0].hHandle, &nFisuCount0, &nLssuCount0, &fisu0, &lssu0); DemoCheckRc(g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle, result);

      g_nCurrentFisu0[0] = (fisu0.bib << 7) | fisu0.bsn;
      g_nCurrentFisu0[1] = (fisu0.fib << 7) | fisu0.fsn;
      g_nCurrentFisu0[2] = fisu0.li;
      
      g_nCurrentLssu0[0] = (lssu0.bib << 7) | lssu0.bsn;
      g_nCurrentLssu0[1] = (lssu0.fib << 7) | lssu0.fsn;
      g_nCurrentLssu0[2] = lssu0.li;
      g_nCurrentLssu0[3] = lssu0.sf;

      DemoLogf("current FISU0: ");
      for (i = 0; i < 3; i++) {
        DemoLogf("%.2X ", g_nCurrentFisu0[i]);
      }
      DemoLogf("\n");

      DemoLogf("current LSSU0: ");
      for (i = 0; i < 4; i++) {
        DemoLogf("%.2X ", g_nCurrentLssu0[i]);
      }
      DemoLogf("\n");

      if (OTX_SUCCESS(result)) {
        DemoLogf("current FISUs received: %.10lu  \n", nFisuCount0);
        DemoLogf("current LSSUs received: %.10lu  \n", nLssuCount0);
        DemoLogf("current FISU0 received: bsn = %d, bib = %d, fsn = %d, fib = %d, li = %d\n",
                fisu0.bsn, fisu0.bib, fisu0.fsn, fisu0.fib, fisu0.li);

        DemoLogf("current LSSU0 received: bsn = %d, bib = %d, fsn = %d, fib = %d, li = %d, sf = %d\n",
                lssu0.bsn, lssu0.bib, lssu0.fsn, lssu0.fib, lssu0.li, lssu0.sf);
      }
    }
    break;

  case 'l':
    {
      OtxT1E1Layer1StatusE eLayer1Status;
      OTX_INT32 i;

      for (i = 0; i < g_demoEnv.coLi; i++) {
        DemoLogf("Line status of Li #%d is ", i);
        result = OtxT1E1GetLayer1Status(g_hPhysDevs.hLi[i], &eLayer1Status); DemoCheckRc(g_hPhysDevs.hLi[i], result);
        DemoLogf("%s\n", eLayer1Status == OTX_T1E1_LAYER1_OK ? "OK" : "DOWN");
      }

    }
    break;

  case 'd':
    DemoLogf("Dump DSP debug data\n");
    OtxDrvDbgDump(g_hPhysDevs.hDsp[0], 0);
    break;

  case 'b':
    DemoLogf("Init Bsp\n");
    OtxDrvDbgDump(g_hPhysDevs.hDsp[0], 1);
    break;


  case 't':
    {

      if (g_demoEnv.bUseSenders) {
        // This performs a simulation of MSU and FISU sending. 
        // Li0 is looped to Li1. 
        g_sqDspHdlcReceiver[1].nPrevFsn = 0x7F;
        g_sqDspHdlcReceiver[1].nSs7Errors = 0;

        g_coHdlcRxFrames[1] = 0;
        g_coHdlcRxBytes[1] = 0;
        g_coHdlcBadRxFrames[1] = 0;
        
        OsaThreadStartEx(DemoSs7SimulationFunction, (void *) &g_sqDspHdlcSender[0].nLi);
        
        
        // This performs a simulation of MSU and FISU sending. 
        // Li1 is looped to Li0. 
        g_sqDspHdlcReceiver[0].nPrevFsn = 0x7F;
        g_sqDspHdlcReceiver[0].nSs7Errors = 0;
        g_coHdlcRxFrames[0] = 0;
        g_coHdlcRxBytes[0] = 0;
        g_coHdlcBadRxFrames[0] = 0;
        
        OsaThreadStartEx(DemoSs7SimulationFunction, (void *) &g_sqDspHdlcSender[1].nLi);
        
      }
    }
    break;

  case 'T':
    {

      if (g_demoEnv.bUseSenders) {
        // This performs a simulation of MSU and FISU sending. 
        // Li0 is looped to Li1. 
        g_sqDspHdlcReceiver[1].nPrevFsn = 0x7F;
        g_sqDspHdlcReceiver[1].nSs7Errors = 0;
        g_coHdlcRxFrames[1] = 0;
        g_coHdlcRxBytes[1] = 0;
        g_coHdlcBadRxFrames[1] = 0;
        
        OsaThreadStartEx(DemoSs7SimulationFunction, (void *) &g_sqDspHdlcSender[0].nLi);
        
        
        // This performs a simulation of MSU and FISU sending. 
        // Li1 is looped to Li0. 
        g_sqDspHdlcReceiver[0].nPrevFsn = 0x7F;
        g_sqDspHdlcReceiver[0].nSs7Errors = 0;
        g_coHdlcRxFrames[0] = 0;
        g_coHdlcRxBytes[0] = 0;
        g_coHdlcBadRxFrames[0] = 0;
        
        OsaThreadStartEx(DemoSs7SimulationFunction, (void *) &g_sqDspHdlcSender[1].nLi);
        

        // This performs a simulation of MSU and FISU sending. 
        // Li2 is looped to Li3. 
        g_sqDspHdlcReceiver[3].nPrevFsn = 0x7F;
        g_sqDspHdlcReceiver[3].nSs7Errors = 0;
        g_coHdlcRxFrames[3] = 0;
        g_coHdlcRxBytes[3] = 0;
        g_coHdlcBadRxFrames[3] = 0;
        
        OsaThreadStartEx(DemoSs7SimulationFunction, (void *) &g_sqDspHdlcSender[2].nLi);
        
        
        // This performs a simulation of MSU and FISU sending. 
        // Li3 is looped to Li2. 
        g_sqDspHdlcReceiver[2].nPrevFsn = 0x7F;
        g_sqDspHdlcReceiver[2].nSs7Errors = 0;
        g_coHdlcRxFrames[2] = 0;
        g_coHdlcRxBytes[2] = 0;
        g_coHdlcBadRxFrames[2] = 0;
        
        OsaThreadStartEx(DemoSs7SimulationFunction, (void *) &g_sqDspHdlcSender[3].nLi);
        

        // This performs a simulation of MSU and FISU sending. 
        // Li4 is looped to Li5. 
        g_sqDspHdlcReceiver[5].nPrevFsn = 0x7F;
        g_sqDspHdlcReceiver[5].nSs7Errors = 0;
        g_coHdlcRxFrames[5] = 0;
        g_coHdlcRxBytes[5] = 0;
        g_coHdlcBadRxFrames[5] = 0;
        
        OsaThreadStartEx(DemoSs7SimulationFunction, (void *) &g_sqDspHdlcSender[4].nLi);
        
        
        // This performs a simulation of MSU and FISU sending. 
        // Li5 is looped to Li4. 
        g_sqDspHdlcReceiver[4].nPrevFsn = 0x7F;
        g_sqDspHdlcReceiver[4].nSs7Errors = 0;
        g_coHdlcRxFrames[4] = 0;
        g_coHdlcRxBytes[4] = 0;
        g_coHdlcBadRxFrames[4] = 0;
        
        OsaThreadStartEx(DemoSs7SimulationFunction, (void *) &g_sqDspHdlcSender[5].nLi);
        


        // This performs a simulation of MSU and FISU sending. 
        // Li6 is looped to Li7. 
        g_sqDspHdlcReceiver[7].nPrevFsn = 0x7F;
        g_sqDspHdlcReceiver[7].nSs7Errors = 0;
        g_coHdlcRxFrames[7] = 0;
        g_coHdlcRxBytes[7] = 0;
        g_coHdlcBadRxFrames[7] = 0;
        
        OsaThreadStartEx(DemoSs7SimulationFunction, (void *) &g_sqDspHdlcSender[6].nLi);
        
        
        // This performs a simulation of MSU and FISU sending. 
        // Li7 is looped to Li6. 
        g_sqDspHdlcReceiver[6].nPrevFsn = 0x7F;
        g_sqDspHdlcReceiver[6].nSs7Errors = 0;
        g_coHdlcRxFrames[6] = 0;
        g_coHdlcRxBytes[6] = 0;
        g_coHdlcBadRxFrames[6] = 0;
        
        OsaThreadStartEx(DemoSs7SimulationFunction, (void *) &g_sqDspHdlcSender[7].nLi);
        
      }
    }
    break;



  case 'g':
    {

      if (g_demoEnv.bUseSenders) {
        // This performs a simulation of MSU and FISU sending. 
        // Li0 is looped to Li1. 
        g_sqDspHdlcReceiver[1].nPrevFsn = 0x7F;
        g_coHdlcRxFrames[1] = 0;
        g_coHdlcRxBytes[1] = 0;
        g_coHdlcBadRxFrames[1] = 0;
        
        OsaThreadStartEx(DemoSs7SimulationFunction, (void *) &g_sqDspHdlcSender[0].nLi);
        
      }
    }
    break;


  case 'F':
    {
      OTX_HANDLE hHandle;
      OTX_UINT32 filterMask;

      hHandle = g_sqDspHdlcReceiver[nCurrentHdlcDev].hHandle;

      result = OtxHdlcSS7GetFilter(hHandle, &filterMask); DemoCheckRc(hHandle, result);
      if (filterMask & OTX_HDLC_SS7_FILTER_FISU_ALL) {
        filterMask &= ~OTX_HDLC_SS7_FILTER_FISU_ALL;
      } else {
        filterMask |= OTX_HDLC_SS7_FILTER_FISU_ALL;
      }

      DemoLogf("Setting SS7 filter mask to %.4X for HDLC Receiver #%d (Dsp#%d, Li#%d)...", filterMask, nCurrentHdlcDev, g_sqDspHdlcReceiver[nCurrentHdlcDev].nDsp, g_sqDspHdlcReceiver[nCurrentHdlcDev].nLi);

      result = OtxHdlcSS7SetFilter(hHandle, filterMask); DemoCheckRc(hHandle, result);

      DemoLogf("Done.\n");
    }
    break;

  case 'f':
    {
      if (g_demoEnv.bUseSenders) {
        OTX_HANDLE hHandle;
        
        hHandle = g_sqDspHdlcSender[nCurrentHdlcDev].hHandle;
        DemoLogf("Start Sending FISUs (fsn=%d) continously from HDLC Sender #%d (Dsp#%d, Li#%d)...", nCurrentHdlcDev, g_sqDspHdlcSender[nCurrentHdlcDev].nDsp, g_sqDspHdlcSender[nCurrentHdlcDev].nLi);
        
        if (nCurrentHdlcDev == 0) {
          g_tFisu0.fsn++;
          DemoLogf("g_tFisu0.bsn=%d, g_tFisu0.bib = %d, g_tFisu0.fsn=%d, g_tFisu0.fib = %d\n", g_tFisu0.bsn, g_tFisu0.bib, g_tFisu0.fsn, g_tFisu0.fib); 
          result = OtxHdlcStartRepeatData(hHandle, (OTX_UINT8 *) &g_tFisu0, OTX_HDLC_SS7_CB_FISU, DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(hHandle, result);
        } else {
          g_tFisu1.fsn++;
          result = OtxHdlcStartRepeatData(hHandle, (OTX_UINT8 *) &g_tFisu1, OTX_HDLC_SS7_CB_FISU, DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(hHandle, result);
        }
        // Note: The following action merely demonstrates the use of the OTX SS7 API functions. It does not necessarily demonstrate proper SS7 protocol procedures
        
        DemoLogf("Done.\n");
      }
    }
    break;

  case 'z':
    {
      if (g_demoEnv.bUseSenders) {
        OTX_HANDLE hHandle;
        
        hHandle = g_sqDspHdlcSender[nCurrentHdlcDev].hHandle;
        DemoLogf("Start Sending FISUs (fsn=%d) continously from HDLC Sender #%d (Dsp#%d, Li#%d)...", nCurrentHdlcDev, g_sqDspHdlcSender[nCurrentHdlcDev].nDsp, g_sqDspHdlcSender[nCurrentHdlcDev].nLi);
        
        if (nCurrentHdlcDev == 0) {
          g_tFisu0.bsn = 0x7F;
          g_tFisu0.bib = 1;
          g_tFisu0.fsn = 0;
          g_tFisu0.fib = 0;
          DemoLogf("g_tFisu0.bsn=%d, g_tFisu0.bib = %d, g_tFisu0.fsn=%d, g_tFisu0.fib = %d\n", g_tFisu0.bsn, g_tFisu0.bib, g_tFisu0.fsn, g_tFisu0.fib); 
          result = OtxHdlcStartRepeatData(hHandle, (OTX_UINT8 *) &g_tFisu0, OTX_HDLC_SS7_CB_FISU, DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(hHandle, result);
        } else {
          g_tFisu1.bib = 1;
          g_tFisu1.fib = 1;
          g_tFisu1.fsn = 0x7F;
          g_tFisu1.bsn = 0x7F;
          result = OtxHdlcStartRepeatData(hHandle, (OTX_UINT8 *) &g_tFisu1, OTX_HDLC_SS7_CB_FISU, DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(hHandle, result);
        }
        
        DemoLogf("Done.\n");
      }
    }
    break;

  case 'i':
    {
      if (g_demoEnv.bUseSenders) {
        OTX_HANDLE hHandle;
        
        hHandle = g_sqDspHdlcSender[nCurrentHdlcDev].hHandle;
        
        // Note: The following action merely demonstrates the use of the OTX SS7 API functions. It does not necessarily demonstrate proper SS7 protocol procedures
        g_tLssu.fsn = g_tFisu0.fsn;
        g_tLssu.sf = g_tLssu.sf + 1; 
        g_tLssu.li = 1; // Short LSSU
        
        g_tLssu.bsn = 0x7f;
        g_tLssu.bib = 0x1;
        
        DemoLogf("g_tLssu.bsn=%d, g_tLssu.bib = %d, g_tLssu.fsn=%d, g_tLssu.fib = %d\n", g_tLssu.bsn, g_tLssu.bib, g_tLssu.fsn, g_tLssu.fib); 
        DemoLogf("Start Sending LSSUs (fsn=%d, sf=%d) continously from HDLC Sender #%d (Dsp#%d, Li#%d)...", g_tLssu.fsn, g_tLssu.sf, nCurrentHdlcDev, g_sqDspHdlcSender[nCurrentHdlcDev].nDsp, g_sqDspHdlcSender[nCurrentHdlcDev].nLi);
        
        result = OtxHdlcStartRepeatData(hHandle, (OTX_UINT8 *) &g_tLssu, OTX_HDLC_SS7_CB_SHORT_LSSU, DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(hHandle, result);
        
        DemoLogf("Done.\n");
      }
    }
    break;

  case 'c':
    {
      OtxBrdClockSourceE  eClkSource;
      printf("Set Clock Source:\n");
      printf("0 = OTX_CLOCK_SOURCE_INTERNAL - LIs synchronized to on-board oscillator\n");
      printf("1 = OTX_CLOCK_SOURCE_LOCAL_0  - LIs synchronized to FA0 receive span\n");
      printf("2 = OTX_CLOCK_SOURCE_LOCAL_1  - LIs synchronized to FA1 receive span\n");

      switch (OsaConsoleGetCh() - '0') {
      case 0:
        eClkSource = OTX_CLOCK_SOURCE_INTERNAL;
        break;
      case 1:
        eClkSource = OTX_CLOCK_SOURCE_LOCAL_0;
        break;
      case 2:
        eClkSource = OTX_CLOCK_SOURCE_LOCAL_1;
        break;
      default:
        printf("Invalid entry. Setting Clock Source to OTX_CLOCK_SOURCE_INTERNAL\n");
        eClkSource = OTX_CLOCK_SOURCE_INTERNAL;
        break;
      }

      result = OtxBrdSetClocks(g_hPhysDevs.hThorController0, OTX_CLOCK_MODE_H100, eClkSource); DemoCheckRc(g_hPhysDevs.hThorController0, result);

      // Setting the clock source also performs a reset of the E1T1 devices. Therefore we need to reconfigure the E1T1 devices here.
      DemoConfigureE1T1Devices();

      printf("Done.\n");
    }
    break;
  
  case 'n':
    {
      // Select current nCurrentHdlcDev
      nCurrentHdlcDev++;
      if (nCurrentHdlcDev > (g_demoEnv.coLi*g_demoEnv.coHdlcPerLi)-1) {
        // Wrap 
        nCurrentHdlcDev = 0;
      }
      
      DemoLogf("DSP HDLC Sender#%d (Dsp#%d, Li#%d) Selected \n", nCurrentHdlcDev, g_sqDspHdlcSender[nCurrentHdlcDev].nDsp, g_sqDspHdlcSender[nCurrentHdlcDev].nLi);
      
    }
    break;
    
    
  case 'N':
    {
      // Select current nCurrentHdlcDev
      nCurrentHdlcDev--;
      if (nCurrentHdlcDev < 0) {
        // Wrap
        nCurrentHdlcDev = (g_demoEnv.coLi*g_demoEnv.coHdlcPerLi)-1;
      }
      
      DemoLogf("DSP HDLC Sender#%d (Dsp#%d, Li#%d) Selected \n", nCurrentHdlcDev, g_sqDspHdlcSender[nCurrentHdlcDev].nDsp, g_sqDspHdlcSender[nCurrentHdlcDev].nLi);
    }
    break;

  case 'I':
    {
      OTX_UINT32 readVal;
      char strBuf[12];
      static OTX_UINT32 nAddress = 0;
      OTX_INT32 nCurrentLi = 0;

      DemoLogf("Read from an 8-bit register in Li%d \n", nCurrentLi);
      DemoLogf("Which Address (0x%X): ", nAddress);

      OsaConsoleGetStr(strBuf); 
      if (*strBuf) {
        nAddress = strtoul(strBuf, NULL, 0);
      }

      result = OtxDrvReadReg8(g_hPhysDevs.hLi[nCurrentLi], nAddress, &readVal); DemoCheckRc(g_hPhysDevs.hLi[nCurrentLi], result);

      DemoLogf("Li%d Address 0x%X = %.2X\n", nCurrentLi, nAddress, (OTX_UINT8)readVal);
    }
    break;

  case 'O':
    {
      char strBuf[12];
      static OTX_UINT32 nAddress = 0;
      static OTX_UINT32 nValue = 0;
      OTX_INT32 nCurrentLi = 0;

      DemoLogf("Write to an 8-bit register in Li%d \n", nCurrentLi);
      DemoLogf("Which Address (0x%X): ", nAddress);
      OsaConsoleGetStr(strBuf);  
      if (*strBuf) {
        nAddress = strtoul(strBuf, NULL, 0);
      }
      DemoLogf("Which Value (0x%X): ", nValue);
      OsaConsoleGetStr(strBuf);   
      if (*strBuf) {
        nValue = strtoul(strBuf, NULL, 0);
      }

      result = OtxDrvWriteReg8(g_hPhysDevs.hLi[nCurrentLi], nAddress, nValue); DemoCheckRc(g_hPhysDevs.hLi[nCurrentLi], result);

      DemoLogf("Wrote %.2X to Li%d Address 0x%X\n", (OTX_UINT8)nValue, nCurrentLi, nAddress);
    }
    break;


    
        
  case '~':
    {
      char strBuf[12];
      OTX_INT32 nSleepTime;

      printf("Enter milliseconds to sleep :");
      OsaConsoleGetStr(strBuf);

      if (strBuf[0]) {
        nSleepTime = strtol(strBuf, NULL, 0);
        printf("Sleeping %d milliseconds\n", nSleepTime);
        OsaThreadSleep(nSleepTime);
      }
    }
    break;

  case 'q':
  case 'Q':
    return 1;
    break;

  default:
    DemoLogf("Unknown command. Press '?' for help, 'q' to quit\n");
    break;
  }

  return 0;
}


/*******************************************************************************
*******************************************************************************/
int main(int argc, char *argv[])
{
  OTX_INT32 exitFlag = 0;
  OTX_RESULT result;  // OTX Standard Return Code
  char commandChar;

	OsaConsoleInit();

  OsaMutexInit(&g_demoLogfCritSec);
  OsaMutexInit(&g_demoSendSs7DataCritSec);

  DemoLogf("T2p5x4Ss7, %s, Odin TeleSystems Inc.\n", __DATE__);
  DemoLogf("SS7 Example program.\n");
  DemoParseCmdLineArgs(argc, argv, &g_demoEnv);

  if (g_demoEnv.pszInputStr) {
    OsaConsoleInput(g_demoEnv.pszInputStr);
  }

  if ((g_pDemoLogFile = fopen(g_demoEnv.logFileName, "wt")) == NULL) {
    printf("Couldn't open logfile %s\n", g_demoEnv.logFileName);
    OsaConsoleExit();
		exit(-1);
  }

  if ((g_pFrameRxFile = fopen("T2p5x4Ss7RxFrames.log", "wt")) == NULL) {
    printf("Couldn't open logfile %s\n", "T2p5x4Ss7RxFrames.log");
    OsaConsoleExit();
		exit(-1);
  }


  DemoLogf("Log file is %s\n", g_demoEnv.logFileName);
  DemoLogf("Board number %d\n", g_demoEnv.boardNo);

  // Query Library revision
  {
    char szVersionInfo[255];
    OTX_BOOL bIsStaticLinkedLibrary;
    char szLibFileName[64];
    result = OtxDrvGetAttributeValueSz(OTX_LIB_HANDLE, OTX_ATTR_LIB_REV_STR, szVersionInfo, 255); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    result = OtxDrvGetAttributeValueBool(OTX_LIB_HANDLE, OTX_ATTR_LIB_TYPE_STATIC, &bIsStaticLinkedLibrary); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    result = OtxDrvGetAttributeValueSz(OTX_LIB_HANDLE, OTX_ATTR_LIB_FILE_NAME, szLibFileName, 64); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    
    DemoLogf("Library version %s (filename %s, %s linked)\n", szVersionInfo, szLibFileName, bIsStaticLinkedLibrary ? "static":"dynamic");
  }

  {
    char *pszClockSource;

    switch (g_demoEnv.eClkSource) {
    case OTX_CLOCK_SOURCE_INTERNAL:
      pszClockSource = "Internal";
      break;
    case OTX_CLOCK_SOURCE_LOCAL_0:
      pszClockSource = "Line Interface 0";
      break;
    case OTX_CLOCK_SOURCE_LOCAL_1:
      pszClockSource = "Line Interface 1";
      break;
    case OTX_CLOCK_SOURCE_LOCAL_2:
      pszClockSource = "External";
      break;
    case OTX_CLOCK_SOURCE_LOCAL_3:
      pszClockSource = "Split Clock, both Line Interface are synchronized to its T1E1 span";
      break;
    case OTX_CLOCK_SOURCE_LOCAL_4:
      pszClockSource = "Split Clock, Li0 synchronized to its T1E1 span. Li1 internal";
      break;
    case OTX_CLOCK_SOURCE_LOCAL_5:
      pszClockSource = "Split Clock, Li1 synchronized to its T1E1 span. Li0 internal";
      break;
    default:
      pszClockSource = "Unknown";
      break;
    }

    DemoLogf("Log file is %s\n", g_demoEnv.logFileName);
    DemoLogf("Clock Source: %s\n", pszClockSource);
  }


  // Connect to the Driver
  DemoLogf("Connecting lib...\n");
  result = OtxDrvConnectLib(); 
  switch (OTX_RESULT_CODE(result)) {
  case OTX_RES_HANDLE:
    DemoLogf("Error: Could not connect to the OTX library.\nPossible cause: OTX Driver not loaded\n");
    OsaConsoleExit();
    OtxDrvReleaseLib();
    exit(1);
    break;
  case OTX_RES_NOT_SUPPORTED:
    DemoLogf("Error: Mismatch between OTX Driver revision and OTX Library revision.\nRevisions must match.\n");
    OsaConsoleExit();
    OtxDrvReleaseLib();
    exit(1);
    break;
  default:
    DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    break;
  }

  // Create Event Queues for Driver --> Application Communications
  DemoLogf("Creating event queue...\n");
  DemoCreateEventQueue();
  DemoLogf("Created event queue\n");

  // Connect (get handles) to the physical devices on the Thor-2-PCI board
  DemoLogf("Opening physical devices...\n");
  DemoOpenPhysicalDevices();
  DemoLogf("Opened physical devices\n");

  // Configure E1/T1 devices
  DemoLogf("Configuring E1/T1 devices...\n");
  DemoConfigureE1T1Devices();
  DemoLogf("Configured E1/T1 devices\n");

  // Enable all physical devices. Their device state should now be OTX_DEVICE_WORKING
  DemoLogf("Enabling physical devices...\n");
  DemoEnablePhysicalDevices();

  // Create and get handles to the Logical devices needed by this application
  DemoLogf("Creating logical devices...\n");
  DemoCreateLogicalDevices();
  DemoLogf("Created logical devices\n");

  // Create and get handles to the Pipe devices needed by this application
  DemoLogf("Connecting devices...\n");
  DemoConnectDevices();

  // Initialized the Connected Logical Devices
  DemoLogf("Initializing devices...\n");
  DemoInitializeLogicalDevices();

  // Set clock source 
  if (g_hPhysDevs.hVidarAsmController0) {
    // Thor-PCI
    result = OtxBrdSetClocks(g_hPhysDevs.hThorController0, OTX_CLOCK_MODE_LOCAL, g_demoEnv.eClkSource); DemoCheckRc(g_hPhysDevs.hThorController0, result);
#if 1  // Temporary
    {
      OTX_INT32 nDsp;
      for (nDsp = 0; nDsp < g_demoEnv.coDsp; nDsp++) {
        DemoLogf("Init Bsp for Dsp#%d\n", nDsp);
        OtxDrvDbgDump(g_hPhysDevs.hDsp[nDsp], 1);
      }
    }
#endif
  } else {
    // Thor-2-PCMCIA
    result = OtxBrdSetClocks(g_hPhysDevs.hThorController0, OTX_CLOCK_MODE_NA, g_demoEnv.eClkSource); DemoCheckRc(g_hPhysDevs.hThorController0, result);
  }

  // Enable all logical devices. Their device state should now be OTX_DEVICE_WORKING
  DemoLogf("Enabling logical devices...\n");
  DemoEnableLogicalDevices();

  // Read Serial Number from the board
  {
    OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
    OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
    OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
    char *pszSerialNumber;

    result = OtxDrvGetAttributeType(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_SERIAL_NUMBER, &eAttribType, &cbAttribValue, &bReadOnly); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    assert(eAttribType == OTX_ATTRIB_SZ);
    assert(bReadOnly == OTX_TRUE);
    pszSerialNumber = (char *) malloc(cbAttribValue * sizeof(char));
    result = OtxDrvGetAttributeValueSz(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_SERIAL_NUMBER, pszSerialNumber, cbAttribValue); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    DemoLogf("Board Serial Number: %s\n", pszSerialNumber);
    free(pszSerialNumber);
  }

  // Read Build Number from the board
  {
    OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
    OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
    OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
    char *pszBrdBuildNumber;

    result = OtxDrvGetAttributeType(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_BUILD, &eAttribType, &cbAttribValue, &bReadOnly); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    assert(eAttribType == OTX_ATTRIB_SZ);
    assert(bReadOnly == OTX_TRUE);
    pszBrdBuildNumber = (char *) malloc(cbAttribValue * sizeof(char));
    result = OtxDrvGetAttributeValueSz(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_BUILD, pszBrdBuildNumber, cbAttribValue); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    DemoLogf("Board Build Number: %s\n", pszBrdBuildNumber);
    free(pszBrdBuildNumber);
  }

  if (g_demoEnv.nBoardType != OTX_DEVICE_THOR_PCMCIA) {
    // Read CPLD Build Numbers from the board
    {
      OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
      OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
      OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
      char *pszCpldBuildNumbers;
      
      result = OtxDrvGetAttributeType(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_CPLD_BUILD, &eAttribType, &cbAttribValue, &bReadOnly); DemoCheckRc(g_hPhysDevs.hThorController0, result);
      assert(eAttribType == OTX_ATTRIB_SZ);
      assert(bReadOnly == OTX_TRUE);
      pszCpldBuildNumbers = (char *) malloc(cbAttribValue * sizeof(char));
      result = OtxDrvGetAttributeValueSz(g_hPhysDevs.hThorController0, OTX_ATTR_BRD_CPLD_BUILD, pszCpldBuildNumbers, cbAttribValue); DemoCheckRc(g_hPhysDevs.hThorController0, result);
      DemoLogf("CPLD Build:\n%s", pszCpldBuildNumbers);
      free(pszCpldBuildNumbers);
    }
  }
  
  DemoLogf("\nPress 'h' for help\n");

  // Main Loop. Loop here and wait for Events from the driver or commands from the user ....
	OsaThreadStart(DemoIdleFunction);

#if 1
  while (!exitFlag) {
    commandChar = OsaConsoleGetCh();
    exitFlag = DemoProcessUserCommand(commandChar);
  }

  if (exitFlag) {
    DemoLogf("\nexitFlag == TRUE.... Exiting\n");
  }
#else // Just for testing. Exiting automatically after 300ms
  OsaThreadSleep(300);
#endif

  // Clean up
  OtxDrvReleaseLib();
	OsaConsoleExit();

  return 0;
}

#if defined(_WIN32_WCE)
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nShowCmd) 
{
  // default command line parameters
  WCHAR wsExeName[OTX_MAX_PATH * sizeof(WCHAR)];
  static char exename[OTX_MAX_PATH];
  static char *cmdParams[] = { exename, "/TM", NULL }; 
  char *pc;

  GetModuleFileName(NULL, wsExeName, sizeof(wsExeName));
  WideCharToMultiByte(CP_ACP, 0, &wsExeName[0], sizeof(wsExeName), exename, sizeof(exename), NULL, NULL); 
  OsaStrCpy(g_demoEnv.cwd, cmdParams[0]);
  pc = OsaStrRChr(g_demoEnv.cwd, '\\');
  if (pc != NULL) {
    *++pc = '\0';
  }

  return OsaWinCeConsoleMain(hInstance, hPrevInstance, lpCmdLine, nShowCmd,
              cmdParams, sizeof(cmdParams) / sizeof(cmdParams[0]), main);
}
#endif  // #if defined(_WIN32_WCE)
