/* $Workfile: OtxDmp.h $   $Revision: 1 $    $Modtime: 10/22/07 12:06p $  */
#ifndef OTX_DMP_H
#define OTX_DMP_H
/*******************************************************************************

  @doc PublicHwApi OtxDmp

  Copyright (C) 2007 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxDmp | This file contains prototypes for the Digital Media Processor (DMP) Management Functions

  @contents2
  OtxDmp.h |

  @index func | OtxDmp

  @contents2
  OTX DMP Device Management Functions - OtxDmpXXX() | The user functions available for
  the Management of GPP ARM Devices

  @head3 OtxDmpXXX() Function Reference |

  @index func | OtxDmp

*******************************************************************************/

#include "OtxType.h"    // OTX specific type definitions
#include "OtxDef.h"     // OTX specific definitions

#include "OtxDmpD.h"    // OTX DMP type definitions

#ifdef __cplusplus
extern "C" {
#endif



/*******************************************************************************

  @func
    Load and Run a Codec Engine Server on a ARM from im formatted program file

  @ex |

    // open physical device of DMP 
    for (i = 0; i < 2; i++) {
      result = OtxDrvOpenPhysicalDevice(
                  g_hPhysDevs.hBoardController, // Vidar series or Thor-8-PCI-Plus series 
                  OTX_DEVICE_DMP, 
                  i, 
                  i, 
                  g_hEventQueue, 
                  &g_hPhysDevs.hDmp[i]); 
    }
    
    // load DMP program from file OtxSpm1.img
    for (nDmp = 0; nDmp < 2; nDmp++) {
      result = OtxDmpRunCodecEngineServer(
                  g_hPhysDevs.hDmp[nDmp], 
                  "OtxSpm1.tgz", 
                  "otx.ce.spm1"); 
    }

  @xref
    <f OtxDmpRunSpm>
    <f OtxDmpRestartProgram>
********************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpRunCodecEngineServer(
  IN OTX_HANDLE     hDmpDevice,         // @parm Handle to DMP device
  IN const char     *szPrgfile,         // @parm Application file to be loaded
  IN const char     *szEntryPoint       // @parm Label for the entry point of the application
);


/*******************************************************************************

  @func
    Load and Run an OTX Standard Program Module (SPM) on a DMP from the a 
    program image stored in the OTX lib.
  
  @ex |
  
    // open physical device of DMP 
    for (i = 0; i < 2; i++) {
      result = OtxDrvOpenPhysicalDevice(
                  g_hPhysDevs.hBoardController, // Vidar series or Thor-2-PCMCIA series 
                  OTX_DEVICE_DMP, 
                  i, 
                  i, 
                  g_hEventQueue, 
                  &g_hPhysDevs.hDmp[i]); 
    }
  
  
    // load DMP Program OTX_DMP_SPM_OTXSPM1
    for (i = 0; i < 2; i++) {
      result = OtxDmpRunStandardProgramModule(
                  g_hPhysDevs.hDmp[i], 
                  OTX_DMP_SPM_OTXSPM1); 
    }

  @xref
    <f OtxDmpRunProgram>
    <t OtxDmpSpmE>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpRunStandardProgramModule(
  IN OTX_HANDLE     hDmpDevice,         // @parm Handle to DMP device
  IN OtxDmpSpmE     eProgramModule      // @parm Standard Program Module (SPM) to be loaded in the DMP (from the Lib)
);


/*******************************************************************************

  @func
    Load and start a DMP application program. The DMP application program must 
    be in the OTX DMP Image format (created by a call 
    to <f OtxDmpCreateProgramImage>). 

  @ex |
    see example on       
    @ex <f OtxDmpCreateProgramImage> |
    @ex <f OtxDmpCreateProgramImageEx> |

  @xref
    <f OtxDmpCreateProgramImage>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpRunCodecEngineServerImage(
  IN OTX_HANDLE     hDmpDevice,         // @parm Handle to DMP device
  IN OTX_UINT16     *psqProgImage,      // @parm Program image 
  IN OTX_UINT16     coProgImage,        // @parm Number of elements in psqProgImage 
  IN OTX_INT32      nEntryAddr          // @parm Program Entry Address 
);


/*******************************************************************************

  @func
    DMP Debugging utility function. Restarts a previously loaded and/or 
    running program. 
    
    This function is only meant to be called if a DMP Debugger is being
    used. 

  @ex |

    // load DMP program
    for (nDmp = 0; nDmp < 2; nDmp++) {
      result = OtxDmpRunSpmCodecEngineServer(
                  g_hPhysDevs.hDmp[nDmp], 
                  OTX_DMP_SPM_OTXSPM1); 
    }

    // load and run DMP program to all of the DMPs in the DMP Debugger

    // restart program on DMPs
    for (i = 0; i < 2; i++) {
        result = OtxDmpRestartCES(g_hPhysDevs.hDmp[i]); 
    }

  @xref
    <f OtxDmpRunProgram>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpRestartCodecEngineServer(
  IN OTX_HANDLE     hDmpDevice          // @parm Handle to DMP device
);


/*******************************************************************************

  @func
    Send a user command to a user supplied DMP application

  @ex |

      Demo1IoInSetBlinkFreqS sIn;
      OTX_UINT32 nBytesReturned;
      OTX_UINT16 nDemo1Rc;
      
      sIn.nBlinkCountFactor = DEMO_FAST_BLINK_COUNT_FACTOR;
      sIn.nDummyData = 6;
      
      // sending Blink Count factor DEMO_FAST_BLINK_COUNT_FACTOR to User App 
      result = OtxDmpIoControl(
                  g_hLogicDevs.hUserApp, 
                  DEMO1_IOCTL_SET_BLINK_FREQ, 
                  &sIn, 
                  sizeof(sIn), 
                  &nDemo1Rc, 
                  sizeof(nDemo1Rc), 
                  &nBytesReturned, 
                  OTX_TASK_SYNC, 
                  OTX_TIME_INFINITY); 

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpIoControl(
  IN  OTX_HANDLE    hDmpDevice,         // @parm Handle to the physical DMP device
  IN  OTX_UINT16    nIoControlCode,     // @parm I/O Control Code to be passed 
  IN  void          *pInBuf,            // @parm Pointer to the user allocated input buffer
  IN  OTX_UINT32    nInBufSize,         // @parm Size of the input buffer
  IN  void          *pOutBuf,           // @parm Pointer to the user allocated output buffer
  IN  OTX_UINT32    nOutBufSize,        // @parm Size of the output buffer
  OUT OTX_UINT32    *pnBytesReturned,   // @parm Number of Bytes returned
  IN  OTX_TASK_REF  nTaskRef,           // @parm See <t OTX_TASK_REF>
  IN  OTX_TIME      nWaitMaxMs          // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Read data from a user supplied DMP application

  @comm
    Usually it is used in event case OTX_LDEVICE_USER_APPLICATION of user 
    DMP application

  @rdesc Returns one of the following values:

    @flag OTX_S_OK | Data was fetched
    @flag OTX_S_FALSE | No Data available at this moment
    @flag OTX_E_BUFFER_TO_SMALL | The supplied user data buffer is too small to fit the received data

  @ex |
    // Note: function DemoLogf() is used to display message in our demo programs 

    case OTX_LDEVICE_USER_APPLICATION:
      if (pEventData->m_nCode == OTX_USER_APPLICATION_EC_READ_DATA) {
        OTX_UINT8  buf[8192];
        OTX_UINT32 nBytesReturned;
        OTX_UINT32 nIoControlCode;

        otxRc = OtxDmpReadData(
                   pEventData->m_hDevice, 
                   OTX_TASK_SYNC, 
                   buf, 
                   8192, 
                   &nBytesReturned, 
                   &nIoControlCode);
        
        if (OTX_RESULT_CODE(otxRc) == OTX_S_OK) {
          switch(nIoControlCode) {
            case DEMO1_NOTIFY_DEBUG_DATA:
            {
              Demo1NotifyGetDebugDataS *pDataS = (Demo1NotifyGetDebugDataS *)buf;
              if (nBytesReturned != sizeof(Demo1NotifyGetDebugDataS)) {
                DemoLogf("Unexpected data length for this IoControlCode (nIoControlCode = %d, nBytesReturned=%d)\n", nIoControlCode, nBytesReturned);
              } else {
                DemoLogf("DEMO1_NOTIFY_DEBUG_DATA (nRequestId=%d, nBytes=%d)\n", pEventData->m_nRequestId, nBytesReturned);
                DemoLogf("nData0 = %.4X\n", pDataS->nData0);
                DemoLogf("nData1 = %.4X\n", pDataS->nData1);
                DemoLogf("nData2 = %.4X\n", pDataS->nData2);

                DemoLogf("sqBuf[299] = %.4X\n", pDataS->sqBuf[299]);
              }
            }
            break;
        
            default:
              DemoLogf("Unknown IoControlCode received in OtsDmpReadData (nIoControlCode = %d, nBytesReturned=%d, nRequestId=%d)\n", nIoControlCode, nBytesReturned, pEventData->m_nRequestId);
            break;
          }
      
        } else {
        
          OTX_RESULT errorCode = otxRc;
          otxRc = OtxDrvResultCode2String(
                     pEventData->m_hDevice, 
                     errorCode, 
                     256, 
                     &msgStr[0]);
          
          if (OTX_SUCCESS(otxRc)) {
            DemoLogf("User App[%d] Error: %s \n", pEventData->m_nSourceId, msgStr);
          } else {
            DemoLogf("User App[%d] Invalid Error Code: %ld\n", pEventData->m_nSourceId, errorCode);
          }
        }

      } else {

        otxRc = OtxDrvEventCode2String(
                   pEventData->m_hDevice, 
                   pEventData->m_nCode, 
                   256, 
                   &msgStr[0]);

        if (OTX_SUCCESS(otxRc)) {
          DemoLogf("User App[%d] Event: %s (nRequestId=%d)\n", pEventData->m_nSourceId, msgStr, pEventData->m_nRequestId);
        } else {
          DemoLogf("User App[%d] Invalid Event: %ld (nRequestId=%d)\n", pEventData->m_nSourceId, pEventData->m_nCode, pEventData->m_nRequestId);
        }
      }
    break;  // end of OTX_LDEVICE_USER_APPLICATION

  @xref

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpReadData(
  IN  OTX_HANDLE    hDmpDevice,         // @parm Handle to the physical DMP device
  IN  OTX_TASK_REF  nTaskRef,           // @parm See <t OTX_TASK_REF>
  OUT OTX_UINT8     *pOutBuf,           // @parm Pointer to the user allocated output buffer
  IN  OTX_UINT32    nOutBufSize,        // @parm Size of the output buffer
  OUT OTX_UINT32    *pnBytesReturned,   // @parm Number of Bytes returned
  OUT OTX_UINT32    *pnIoControlCode    // @parm I/O Control Code the returned data is associated with
);


/*******************************************************************************

  @func
    Read a number of octets from the DMP's Host Port Interface (HPI)
    into a user supplied buffer.

  @comm
    Note: This function should be used with caution. It is intended for
    debugging purposes and advanced users only.

    API <f OtxDmpWriteHpi> can be used to write the data  

  @ex |
  
    OTX_UINT32 i;
    OTX_UINT16 sqReadBuf[0x800];

    // dump the entire HPI memory for the current DMP
    result = OtxDmpReadHpi(
                g_hPhysDevs.hDmp, 
                0x1000, 
                0x800, 
                sqReadBuf); 

    // display the data
    for (i = 0; i < 0x800; i++) {
        if (i && ((i % 8) == 0)) 
        DemoLogf("\n %.4X ", sqReadBuf[i]);
      }
    }


  @xref
    <f OtxDmpWriteHpi>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpReadHpi(
  IN  OTX_HANDLE    hDmpDevice,         // @parm Handle to the physical DMP device
  IN  OTX_UINT32    nHpiAddress,        // @parm HPI Address to start to read from 
  IN  OTX_UINT32    coWordsToRead,      // @parm Number of OTX_UINT16s (Words) to read from the HPI
  IN OUT OTX_UINT16 *pReadBuf           // @parm Pointer to the user allocated output buffer (Words)
);


/*******************************************************************************

  @func
    Write a number of octets to the DMP's Host Port Interface (HPI)
    from a user supplied buffer.

  @comm
    Note: This function should be used with caution since it can corrupt the
    communication buffers between the DMP and the host. It is intended for
    debugging purposes and advanced users only.

    API <f OtxDmpReadHpi> can be used to check the data written  

  @ex |
  
    static OTX_UINT16 writeVal = 0x5A5A;
    OTX_UINT16 sqWriteBuf[1];

    sqWriteBuf[0] = writeVal;

    // write a value to the last address in the HPI memory for the current DMP
    result = OtxDmpWriteHpi(
                g_hPhysDevs.hDmp, 
                0x17FF, 
                1, 
                sqWriteBuf); 

  @xref
    <f OtxDmpReadHpi>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpWriteHpi(
  IN  OTX_HANDLE    hDmpDevice,         // @parm Handle to the physical DMP device
  IN  OTX_UINT32    nHpiAddress,        // @parm HPI Address to start to write to
  IN  OTX_UINT32    coWordsToWrite,     // @parm Number of OTX_UINT16s (Words) to write to the HPI
  IN OUT OTX_UINT16 *pWriteBuf          // @parm Pointer to the user allocated input buffer (Words)
);

/*******************************************************************************

  @func
    Transfer a file from the host system to the DMP device.

  @xref
    <f OtxDmpDownloadFile>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpUploadFile(
  IN  OTX_HANDLE    hDmpDevice,         // @parm Handle to the physical DMP device
  IN  OTX_UINT8*    szSendFileName,     // @parm File name on the host system to be transmitted
  IN  OTX_UINT8*    szTargetPath       // @parm Target DMP filesystem path
                                        //       i.e "/root"
);

/*******************************************************************************

  @func
    Transfer a file from the DMP device to the host system.

  @xref
    <f OtxDmpUploadFile>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpDownloadFile(
  IN  OTX_HANDLE    hDmpDevice,         // @parm Handle to the physical DMP device
  IN  OTX_UINT8*    szReceiveFileName,  // @parm File name to be received to be placed on the host filesystem
  IN  OTX_UINT8*    szTargetPath        // @parm Target DMP filesystem path
                                        //       i.e "/root"
);

/*******************************************************************************

  @func
    Set the network parameters (e.g. IP address, subnet mask etc) for ethernet port
    of the DMP device.

  @xref
    <f OtxDmpGetNetworkParams>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpSetNetworkParams(
  IN  OTX_HANDLE     hDmpDevice,         // @parm Handle to the physical DMP device
  IN  OTX_UINT8*     szEthDevice,        // @parm File name to be received to be placed on the host filesystem
  IN  OtxDmpNetAddrS sNetworkAddress     // @parm Target DMP Network Address
);

/*******************************************************************************

  @func
    Retrieve the network parameters (e.g. IP address, subnet mask etc) for ethernet port
    of the DMP device.

  @xref
    <f OtxDmpSetNetworkParams>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpGetNetworkParams(
  IN  OTX_HANDLE    hDmpDevice,          // @parm Handle to the physical DMP device
  IN  OTX_UINT8*    szEthDevice,         // @parm File name to be received to be placed on the host filesystem
  OUT OtxDmpNetAddrS   *sNetworkAddress  // @parm Target DMP Network Address
);


/*******************************************************************************

  @func
    Run a separate, stand-alone process in the ARM core of the DMP device.

  @xref
    <f OtxDmpKillStandaloneProcess>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpRunStandaloneProcess(
  IN  OTX_HANDLE    hDmpDevice,         // @parm Handle to the physical DMP device
  IN  OTX_UINT8*    szProcessName,      // @parm Process file name with the path  on the target filesystem
  OUT OTX_UINT32   *pnProcessPID        // @parm Target system Process PID
);

/*******************************************************************************

  @func
    Kill a specific stand-alone process in the ARM core of the DMP device.

  @xref
    <f OtxDmpRunStandaloneProcess>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxDmpKillStandaloneProcess(
  IN  OTX_HANDLE    hDmpDevice,         // @parm Handle to the physical DMP device
  IN  OTX_UINT8*    szProcessName,      // @parm Process file name with the path  on the target filesystem
  IN  OTX_UINT32   *pnProcessPID,       // @parm Target DMP filesystem PID
  IN  OTX_BOOL      bKillAll            // @parm Kill all processes with the same name (PID=0 this case)
);




#ifdef __cplusplus
}  // extern "C"
#endif

/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 2007

  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/

#endif    // OTX_DMP_H
