/* $Workfile: OtxDmpD.h $   $Revision: 1 $    $Modtime: 10/21/07 6:06p $  */
#ifndef OTX_DMP_DEF_H
#define OTX_DMP_DEF_H
/*******************************************************************************

  @doc PublicHwApi OtxDmpD

  Copyright (C) 2007 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxDmpD | Constants, Macros, and Types for the DMP Device Management Functions

  @contents2
  OtxDmpD.h |

  @index enum struct type | OtxDmpD

  @contents2
  OTX DMP Device Management Macros, Constants, and Types |

  @head3 OtxDmpXXX Macro, Constant, and Type Reference |

  @index enum struct type | OtxDmpD

*******************************************************************************/

#include "OtxType.h"    // OTX specific type definitions

#include "OtxErr.h"

#ifdef __cplusplus
extern "C" {
#endif


/*******************************************************************************
  @enum Standard Program Modules (SPM)

  @comm DMP program modules are loaded in the DMPs by calling <f OtxDmpRunSpm>. 

  The number of devices supported for DMP module is per DMP chip (not per board). 


  @comm <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM1> is a legacy SPM and has been 
  superceeded with <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2> and newer SPMs.


  @comm <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2> supports the following logical devices:
    <nl> OTX_LDEVICE_HDLC_SENDER
    <nl> OTX_LDEVICE_HDLC_RECEIVER
    <nl> OTX_LDEVICE_DATA_RAW_SENDER
    <nl> OTX_LDEVICE_DATA_RAW_RECEIVER
    <nl> OTX_LDEVICE_TONE_FSK_DETECTOR
    <nl> OTX_LDEVICE_TONE_DTMF_DETECTOR
    <nl> OTX_LDEVICE_TONE_MF_DETECTOR
    <nl> OTX_LDEVICE_TONE_DUAL_DETECTOR
    <nl> OTX_LDEVICE_TONE_SINGLE_DETECTOR
    <nl> OTX_LDEVICE_TONE_DTMF_DIALER
    <nl> OTX_LDEVICE_TONE_MF_DIALER
    <nl> OTX_LDEVICE_TONE_EFFECTS_GENERATOR
    <nl> OTX_LDEVICE_DATA_CONVERTER
    <nl> OTX_LDEVICE_TONE_CONST_VAL_GENERATOR
    <nl> OTX_LDEVICE_TONE_VAL_SAMPLER  (for c54x based boards)
    <nl> OTX_LDEVICE_TONE_SINEWAVE_GENERATOR
    <nl> OTX_LDEVICE_TONE_SILENCE_DETECTOR

  OtxSpm2 supports pipes of capacity 56kbps, 64kbps and 128kbps.

  The OtxSpm2 is memory limited to 8 devices for the HDLC Sender, 
  HDLC Receiver, Raw Data Sender, Raw Data Receiver. 

  The OtxSpm2 has a buffer size of 102 samples (12.75ms).

  The HDLC Sender and HDLC Receiver devices consume a fair amount of mips,
  thus the mips limitation might be reached before the memory limitation is
  reached.

  The number of other devices (other than HDLC and Raw Data) that the OtxSpm2
  supports varies depending on which DMP is populated on the board. The 
  following list specifies the number of other devices that the OtxSpm2 supports:
    <nl> Thor-2-PCMCIA       - 8  devices, 
    <nl> Thor-2-PCMCIA-Plus  - 8  devices, 
    <nl> Thor-2-PCMCIA-EX    - 20 devices, 
    <nl> Thor-2-PCMCIA-PRO   - 32 devices, 
    <nl> Thor-2-PCMCIA-CST   - 8  devices, 
    <nl> Vidar-5x4-ASM       - 8  devices, 
    <nl> Vidar-5x4-ASM-EX    - 20 devices, 
    <nl> Vidar-5x4-ASM-PRO   - 32 devices, 
    <nl> Vidar-5x4-ASM-CST   - 8  devices, 
    <nl> Vidar-5x16-PCI      - 8  devices,
    <nl> Vidar-55x4-ASM      - 128 devices.
    <nl> Sleipnir-1-PCI-Plus - 32 devices (when equipped with the C5510 DMP).

  For newer OTX boards the <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2> is a legacy SPM and has been 
  superceeded with <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2EX> and <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2EX_INTMEM>.


  @comm <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2EX> supports the same logical devices 
  as <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2>. The added feature
  of OtxSpm2Ex is lower latency HDLC decoding/encoding (1ms in OtxSpm2Ex vs 12.75ms 
  in OtxSpm2). OtxSpm2Ex also supports 16kpbs and 32kpbs subrates for HDLC devices
  (whereas OtxSpm2 only supports 64kpbs HDLC devices). OtxSpm2Ex and OtxSpm2 both have 
  12.75 ms bufferering for tone devices.

  The C5510 based implementation of this DMP program requires 16MB external 
  memory (available on Vidar-55x4-ASM and Sleipnir-1-PCI-Plus).

  OtxSpm2Ex can be run on the following boards:
    <nl> Thor-2-PCMCIA-EX    
    <nl> Thor-2-PCMCIA-PRO  
    <nl> Vidar-5x4-ASM-EX  
    <nl> Vidar-5x4-ASM-PRO  
    <nl> Vidar-55x4-ASM      
    <nl> Sleipnir-1-PCI-Plus (when equipped with the C5510 DMP)


  @comm <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2_INTMEM> is similar to <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2>
  but it is targeted to C5510 based board without external memory. 
   
  It supports pipes of capacity 56kbps (HDLC), 64kbps (HDLC and voice) and 128kbps (voice).

  The OtxSpm2_IntMem is memory limited to 16 devices for the HDLC Sender, 
  HDLC Receiver, Raw Data Sender, Raw Data Receiver. 

  The OtxSpm2_IntMem has a buffer size of 102 samples (12.75ms).

  OtxSpm2_IntMem supports 60 channels of tone devices. Please not that tone devices can be connected
  in parallell. I.e. you can for instance have 60 DTMF detectors, 60 MF Detectors, and 60 SingleTone
  detectors active at one particular time. 16 of these channels can be HDLC devices.

  The OtxSpm2_IntMem is supported on C5510 boards, and does not require any external memory. It runs
  on the internal memory within the C5510 DMP.


  @comm <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2EX_INTMEM> is similar to <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2EX> but 
  it runs all of its code out of the internal memory of the DMP (c5510). It does not require external DMP 
  memory (as <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2EX> does).
    
  Just as in as in <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2EX> the latency for HDLC decoding/encoding is 1ms, and
  the latency for tone devices is 12.75ms.
  
  Just as in as in <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2EX> the OtxSpm2Ex_IntMem also supports 16kpbs 
  and 32kpbs subrates (as well as 56 and 64kpbs) for HDLC devices.
  
  OtxSpm2Ex_IntMem supports 64 channels of tone devices. Please not that tone devices can be connected
  in parallell. I.e. you can for instance have 64 DTMF detectors, 64 MF Detectors, and 64 SingleTone
  detectors active at one particular time. 16 of these channels can be HDLC devices.

  <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2EX_INTMEM> only runs on C5510 based boards. It runs on is the recommended SPM to
  use for these boards:

    <nl> Vidar-55x4-ASM  
    <nl> Sleipnir-1-PCI-Plus (when equipped with the C5510 DMP)
    <nl> Thor-2-ExpressCard


  @comm <e OtxDmpSpmE.OTX_DMP_SPM_OTXHDLC> supports the following logical devices:
    <nl> OTX_LDEVICE_HDLC_RECEIVER 
    <nl> OTX_LDEVICE_HDLC_SENDER   
    <nl> OTX_LDEVICE_DATA_RAW_SENDER
    <nl> OTX_LDEVICE_DATA_RAW_RECEIVER
    <nl> OTX_LDEVICE_DATA_CONVERTER
    
   OtxHdlc supports pipes of capacity 8kbps, 16kbps, 32kbps and n*64kbps.

   The OtxHdlc is memory limited to 32 devices for all of the devices listed above. 
   However, the HDLC Sender and HDLC Receiver devices consume a fair amount of mips,
   thus the mips limitation will be reached before the memory limitation is
   reached. The mips limitation varies depending on which DMP is populated on the 
   board. The following list specifies the approximate number of other devices that
   can be safely be used before the mips limiation is reached:
    <nl> Thor-2-PCMCIA       - 8  devices, 
    <nl> Thor-2-PCMCIA-Plus  - 12 devices, 
    <nl> Thor-2-PCMCIA-EX    - 16 devices, 
    <nl> Thor-2-PCMCIA-PRO   - 16 devices, 
    <nl> Thor-2-PCMCIA-CST   - 12 devices, 
    <nl> Vidar-5x4-ASM       - 8  devices, 
    <nl> Vidar-5x4-ASM-EX    - 16 devices, 
    <nl> Vidar-5x4-ASM-PRO   - 16 devices, 
    <nl> Vidar-5x4-ASM-CST   - 12 devices, 
    <nl> Vidar-5x16-PCI      - 8  devices,
    <nl> Vidar-55x4-ASM      - 32 devices. 
 

  @comm <e OtxDmpSpmE.OTX_DMP_SPM_OTXHDLCEX> supports the following logical devices:
    <nl> OTX_LDEVICE_HDLC_RECEIVER 
    <nl> OTX_LDEVICE_HDLC_SENDER   
    
   OtxHdlcEx supports pipes of capacity n*64kbps.

   The OtxHdlcEx can be run on 5416 and 5510 DMPs and it can handle HDLC pipes
   as wide as 31 timeslots. OtxHdlcEx is memory limited (not MIPS limited). The 
   following limitations apply:

    <nl> Thor-2-PCMCIA-PRO   - 16 devices, 
    <nl> Vidar-55x4-ASM      - 32 devices, 
 

  @comm The <e OtxDmpSpmE.OTX_DMP_SPM_OTXCST2> are only supported by the 
    <nl> Arni-6x6-PCI-Plus (and other Arni-6x6 variants), 
    <nl> Thor-2-PCMCIA-CST,
    <nl> Vidar-5x4-ASM-CST,

   OtxCst2 supports pipes of capacity 64kbps (128kpbs pipe support planned).

   OtxCst2 supports the following logical devices:
    <nl> OTX_LDEVICE_CST_DTMF_DETECTOR   
    <nl> OTX_LDEVICE_CST_DTMF_DIALER         
    <nl> OTX_LDEVICE_CST_VOICE_MIXER         
    <nl> OTX_LDEVICE_CST_CPT_DETECTOR        
    <nl> OTX_LDEVICE_CST_CPT_GENERATOR        
    <nl> OTX_LDEVICE_CST_ECHO_CANCEL
    <nl> OTX_LDEVICE_CST_COMFORT_NOISE_GENERATOR
    <nl> OTX_LDEVICE_CST_VOICE_ACTIVITY_DETECTOR
    <nl> OTX_LDEVICE_CST_CALLER_ID_DETECTOR
    <nl> OTX_LDEVICE_CST_LI_POTS             

  @comm The <e OtxDmpSpmE.OTX_DMP_SPM_OTXCST3> program is only supported by the 
    <nl> Arni-6x6-PCI-Plus (and other Arni-6x6 variants), 
    <nl> Thor-2-PCMCIA-CST,
    <nl> Vidar-5x4-ASM-CST,

   OtxCst3 supports pipes of capacity 64kbps.

   OtxCst3 supports the following logical devices:
    <nl> OTX_LDEVICE_CST_DTMF_DETECTOR   
    <nl> OTX_LDEVICE_CST_DTMF_DIALER         
    <nl> OTX_LDEVICE_CST_VOICE_MIXER         
    <nl> OTX_LDEVICE_CST_CPT_DETECTOR
    <nl> OTX_LDEVICE_CST_CPT_GENERATOR        
    <nl> OTX_LDEVICE_CST_ECHO_CANCEL
    <nl> OTX_LDEVICE_CST_COMFORT_NOISE_GENERATOR
    <nl> OTX_LDEVICE_CST_VOICE_ACTIVITY_DETECTOR
    <nl> OTX_LDEVICE_CST_CALLER_ID_DETECTOR
    <nl> OTX_LDEVICE_HDLC_RECEIVER  
    <nl> OTX_LDEVICE_HDLC_SENDER    
    <nl> OTX_LDEVICE_DATA_RAW_RECEIVER 
    <nl> OTX_LDEVICE_DATA_RAW_SENDER    


*******************************************************************************/
enum OtxDmpSpmE {
  OTX_DMP_SPM_OTXSPM1,          // @emem DMP Program OtxSpm1 (available for C548/C549, C54CST, C5410A, and C5416 DMPs)
  OTX_DMP_SPM_OTXSPM2,          // @emem DMP Program OtxSpm2.out (available for C548/C549, C54CST, C5410A, C5416, and C5510 DMPs)
  OTX_DMP_SPM_OTXSPM2_EXTMEM,   // @emem DMP Program OtxSpm2c549ExtMem.out(available for C548/C549 with 64kword external memory (Vidar-5x4-ASM))
  OTX_DMP_SPM_OTXSPM3,          // @emem DMP Program OtxSpm3.out (No longer supported. Use <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2> instead)
  OTX_DMP_SPM_OTXHDLC,          // @emem DMP Program OtxHdlc.out (available for C548/C549, C54CST, C5410A, C5416, and C5510 DMPs)
  OTX_DMP_SPM_OTXHDLCEX,        // @emem DMP Program OtxHdlcEx.out (available for C5416 and C5510 DMPs)
  OTX_DMP_SPM_OTXDTTX,          // @emem DMP Program OtxDtTx.out (No longer supported. Use <e OtxDmpSpmE.OTX_DMP_SPM_OTXSPM2> instead)
  OTX_DMP_BERT,	                // @emem DMP bert program bert.out (Odin internal)
  OTX_DMP_DIAG,	                // @emem DMP Diagnostics program (Odin internal)
  OTX_DMP_DS0,	                // @emem DMP DS0 Visual program (Odin internal)
  OTX_DMP_SPM_DMPBERT,          // @emem DMP DmpBert program dspbert.out (Odin internal, available for C548/C549 DMPs)
  OTX_DMP_SPM_OTXSPM2EX,        // @emem DMP Program OtxSpm2Ex.out (available for C5410A, and C5416 DMPs)
  OTX_DMP_SPM_OTXCST2,          // @emem DMP Program OtxSpmCst2.out (available for C54CST DMPs; targeted for Arni-6x6-PCI types of boards)
  OTX_DMP_SPM_OTXCST3,          // @emem DMP Program OtxSpmCst3.out (available for C54CST DMPs; targeted for Vidar-CST and PCMCIA-CST boards)
  OTX_DMP_SPM_OTXSPM2_INTMEM,   // @emem DMP Program OtxSpm2c5510IntMem.out. SPM2 functionality for C5510 boards without external memory. Available only for C5510 DMPs.
  OTX_DMP_SPM_OTXSPM2EX_INTMEM, // @emem DMP Program OtxSpm2Exc5510IntMem.out. SPM2 functionality with lowered latency for HDLC devices for C5510 boards without external memory. Available only for C5510 DMPs.
  OTX_DMP_SPM_OTXHDLC_INTMEM,   // @emem DMP Program OtxHdlc5510IntMem.out. HDLC functionality for C5510 boards without external memory. Available only for C5510 DMPs.
};

typedef enum OtxDmpSpmE OtxDmpSpmE;


/******************************************************************************
  @enum 
    DMP Device Attributes
******************************************************************************/
enum OtxDmpDeviceAttributesE {
  OTX_ATTR_DMP_NOTIFICATION_EVENT_BUF_CURRENT_USAGE            = OTX_ATTR_END + 0,  // @emem (OTX_ATTRIB_UINT32) Read-only. Current number of words in use of the 
                                                                                    // HPI Notification Event Buffer
  OTX_ATTR_DMP_NOTIFICATION_EVENT_BUF_MAX_USAGE                = OTX_ATTR_END + 1,  // @emem (OTX_ATTRIB_UINT32) Read-only. Maximum number of words in use of the 
                                                                                    // HPI Notification Event Buffer (since the application started)
  OTX_ATTR_DMP_NOTIFICATION_EVENT_BUF_LOST_EVENTS              = OTX_ATTR_END + 2,  // @emem (OTX_ATTRIB_UINT32) Read-only. Number of the lost events (due to a 
                                                                                    // full HPI Notification Event Buffer)
  OTX_ATTR_DMP_NOTIFICATION_EVENT_EXT_QUEUE_CURRENT_USAGE      = OTX_ATTR_END + 3,  // @emem (OTX_ATTRIB_UINT32) Read-only. Current number of Notification Event packets
                                                                                    // in the extended event queue (located outside of the HPI Notification Buffer)  
  OTX_ATTR_DMP_NOTIFICATION_EVENT_EXT_QUEUE_PEAK_USAGE         = OTX_ATTR_END + 4,  // @emem (OTX_ATTRIB_UINT32) Read-only. Current number of Notification Event packets
                                                                                    // in the extended event queue (located outside of the HPI Notification Buffer)  

  OTX_ATTR_DMP_NOTIFICATION_EVENT_BUF_ADDR                     = OTX_ATTR_END + 5,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Start address of the HPI Notification 
                                                                                    // Event Buffer
  OTX_ATTR_DMP_NOTIFICATION_EVENT_BUF_SIZE                     = OTX_ATTR_END + 6,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Size (in words) of the HPI Notification Event Buffer
  OTX_ATTR_DMP_NOTIFICATION_EVENT_BUF_PROCESSING_ADDR          = OTX_ATTR_END + 7,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Address of where the host driver will 
                                                                                    // write the address of its Notification Event Buffer Read Pointer (i.e. the 
                                                                                    // address of where it will read the next Notification Event from)
  OTX_ATTR_DMP_IOCTL_CODE_ADDR                                 = OTX_ATTR_END + 8,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Start address of the DMP IO Control packet.
  OTX_ATTR_DMP_IOCTL_DATA_MAX_SIZE                             = OTX_ATTR_END + 9,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Maximum data length of the DMP IO Control 
                                                                                    // packet.
  OTX_ATTR_DMP_NOTIFICATION_EVENT_BUF_INCLUDE_HANDLE           = OTX_ATTR_END + 10,  // @emem (OTX_ATTRIB_BOOL) Read/Write. If OTX_TRUE the DMP code must include the 
                                                                                    // DMP Handle in the Notification Events
  OTX_ATTR_DMP_NOTIFICATION_EVENT_BUF_INCLUDE_TASKREF          = OTX_ATTR_END + 11,  // @emem (OTX_ATTRIB_BOOL) Read/Write. If OTX_TRUE the DMP code must include the 
                                                                                    // TaskRef in the Notification Events
  OTX_ATTR_DMP_NOTIFICATION_EVENT_BUF_INCLUDE_TIMESTAMP        = OTX_ATTR_END + 12, // @emem (OTX_ATTRIB_BOOL) Read/Write. If OTX_TRUE the DMP code must include the 
                                                                                    // TimeStamp in the Notification Events
  OTX_ATTR_DMP_IOCTL_INCLUDE_HANDLE                            = OTX_ATTR_END + 13, // @emem (OTX_ATTRIB_BOOL) Read/Write. If OTX_TRUE the host driver will include the 
                                                                                    // DMP Handle in the DMP IO Control packet
  OTX_ATTR_DMP_IOCTL_INCLUDE_TASKREF                           = OTX_ATTR_END + 14, // @emem (OTX_ATTRIB_BOOL) Read/Write. If OTX_TRUE the host driver will include 
                                                                                    // the TaskRef in the DMP IO Control packet
  OTX_ATTR_DMP_AUTO_DISABLE                                    = OTX_ATTR_END + 15, // @emem (OTX_ATTRIB_BOOL) Read/Write. Default OTX_TRUE. If OTX_TRUE the driver 
                                                                                    // will automatically disable the device if the Notification Event buffer encounters
                                                                                    // to many serious errors. A call to <f OtxDrvReset> will reset the error counter for
                                                                                    // serious errors.
                                                              
  OTX_ATTR_DMP_FRAME_BUFFER_COUNTER                            = OTX_ATTR_END + 16, // @emem (OTX_ATTRIB_UINT32) Read only. The Frame Buffer Counter is a 16 bit value
                                                                                    // which is incremented by the DMP code every time it receives another buffer 
                                                                                    // interrupt (highway serial port). Since each DMP is clocked from the same Frame Sync
                                                                                    // clock this counter can be used to synchronize certain events (such as a synchronized 
                                                                                    // enabling of multiple devices). The counter will wrap once it reaches 0xFFFF. The size
                                                                                    // of the frame buffer (auto buffer) is set by each DMP program, but the Frame sync pulse
                                                                                    // always occurs every 125 us.
  OTX_ATTR_DMP_FRAME_COUNTER                                   = OTX_ATTR_END + 17, // @emem (OTX_ATTRIB_UINT32) Read only. The Frame Counter is a 16 bit value
                                                                                    // which is incremented by the DMP code every time it receives another buffer 
                                                                                    // interrupt (highway serial port). It is similar to the 
                                                                                    // <e OtxDmpDeviceAttributesE.OTX_ATTR_DMP_FRAME_BUFFER_COUNTER> attribute except that this
                                                                                    // counter is incremented by the number of frames in each buffer. Therefore this attribute
                                                                                    // can be used to get an accurate count of how many frames that the DMP has received since
                                                                                    // the DMP was started. The time difference between each frame is 125us.
  OTX_ATTR_DMP_HIGHWAY_SPEED                                   = OTX_ATTR_END + 18, // @emem (OTX_ATTRIB_UINT32) Read only. DMP Highway speed in kbps. Valid values are
                                                                                    // 2048, 4096, and 8192. The highway speed determines how many timeslots that the DMP can
                                                                                    // work with. On a 2048 kpbs highway there are 32 timeslots. On a 4096 kbps highway there
                                                                                    // are 64 timeslots, and on an 8192 kpbs highway there are 128 timeslots.

  OTX_ATTR_DMP_HIGHWAY_BIT_FREQUENCY                           = OTX_ATTR_END + 19, // @emem (OTX_ATTRIB_UINT32) Read only. 
                                                                                    // <nl>If supported by the loaded DMP program,
                                                                                    // this attribute returns the bit frequency of the DMP highway in Hertz. 

  OTX_ATTR_DMP_AUTO_LOGICAL_DISCONNECT_DISABLE                 = OTX_ATTR_END + 20, // @emem (OTX_ATTRIB_BOOL) Read/Write. Default OTX_TRUE. If OTX_TRUE the driver 
                                                                                    // will automatically disable the logical device in the DMP if a <f OtxDrvDisconnectLogicalDevice>
                                                                                    // is called for that device. If OTX_FALSE, the DMP skip the disable call, and leave the
                                                                                    // logical device in the current state while disconnecting the logical device.
                                                              
  OTX_ATTR_DMP_END                                             = OTX_ATTR_END + 21
};

typedef enum OtxDmpDeviceAttributesE OtxDmpDeviceAttributesE;


/******************************************************************************
  @enum
    DMP Error Result Codes
******************************************************************************/
enum OtxDmpResultCodesE {
  OTX_DMP_RES_DATA_TOO_LARGE               = OTX_RES_END +  0 ,  // @emem Data too large for the supplied buffer. Supply a larger buffer.
  OTX_DMP_RES_INVALID_CHANNEL              = OTX_RES_END +  1 ,  // @emem Invalid channel
  OTX_DMP_RES_COMM_FAILURE                 = OTX_RES_END +  2 ,  // @emem Communication failure with the DMP
  OTX_DMP_RES_NO_KERNEL_RESPONSE           = OTX_RES_END +  3 ,  // @emem No response from DMP Kernel
  OTX_DMP_RES_NO_FREE_LOGICAL_DEVICE       = OTX_RES_END +  4 ,  // @emem No free logical device

  OTX_DMP_RES_END                          = OTX_RES_END +  1
};

typedef enum OtxDmpResultCodesE OtxDmpResultCodesE;


/******************************************************************************
  @enum
    DMP Event Codes
******************************************************************************/
enum OtxDmpEventCodesE {
  // OtxSpm1 Specific Event Codes
  OTX_DMP_EC_RX_OVERFLOW                 = 0x00, // @emem OtxSpm1 Receive overflow
  OTX_DMP_EC_INTERNAL_ERROR              = 0x01, // @emem OtxSpm1 Undefined Internal error
  OTX_DMP_EC_UNKNOWN_NOTIFY_CODE         = 0x02, // @emem OtxSpm1 Error. Unknown notify code for System Notify Event Handle.

  // Generic DMP Module Event Codes
  OTX_DMP_EC_INVALID_NOTIFY_EVENT_HANDLE = 0x03, // @emem Invalid Notify Event Handle. Could not find a matching logical device for this handle.
  OTX_DMP_EC_NOTIFY_EVENT_OVERFLOW       = 0x04, // @emem Event frequency too high. Driver cannot handle all these events. 
  OTX_DMP_EC_AUTO_DISABLED               = 0x05, // @emem Fatal error has occurred. The device is automatically disabled.
  OTX_DMP_EC_NOTIFY_EVENT_FREQ_HIGH      = 0x06, // @emem Event frequency from the DMP is seriously high. Fatal errors are imminent.
//OTX_DMP_EC_DEBUG_INFO                  = 0x07, // @emem Internal use. For driver debugging.

//*****************************
#ifdef _TURNSTONE_
//*****************************
  OTX_DMP_EC_PATTERN_SYNC                = 0x06,
  OTX_DMP_EC_PATTERN_UNKNOWN             = 0x07,
  OTX_DMP_EC_RXBUFFER_OVERFLOW           = 0x08,
  OTX_DMP_EC_BITERROR_UPDATE             = 0x09,
  OTX_DMP_EC_TEST_PATTERN_FAILURE        = 0x0A,
  OTX_DMP_EC_TEST_PATTERN_SUCCESS        = 0x0B,
  OTX_DMP_EC_DS0VISUAL_UPDATE            = 0x0C,
//******************************
#endif   //_TURNSTONE_
//******************************

  OTX_DMP_END_OF_EVENT_CODES          
};
typedef enum OtxDmpEventCodesE OtxDmpEventCodesE;


/*******************************************************************************
  @struct 
    Data recorded for an event
*******************************************************************************/
struct OtxDmpNetAddrS {
  OTX_UINT32       m_nIpAddress;            // @field IP Address as 0xAA.BB.CC.DD (0xAABBCCDD)
  OTX_UINT32       m_nSubnetMask;           // @field Subnet mask as 0xAA.BB.CC.DD (0xAABBCCDD)
};

typedef struct OtxDmpNetAddrS OtxDmpNetAddrS;




#ifdef __cplusplus
}  // extern "C"
#endif

/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 2007

  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif    // OTX_DMP_DEF_H
